*! version 1.1.0  13may2008
*  asmprobit and asroprobit replay program

program _asprobit_replay
	syntax [, Level(cilevel)]

	local ncov = e(k_rho)+e(k_sigma)
	local ncasecf : word count `"`e(casevars)'"'
	local const = `e(const)' != 0
	local ncasecf = `ncasecf' + `const'

	local noheader noheader 
	local crtype = upper(substr(`"`e(crittype)'"',1,1)) + ///
		substr(`"`e(crittype)'"',2,.)
	di _n in gr `"`e(title)'"' _col(48) "Number of obs" _col(67) "= " ///
	 in ye %10.0g e(N)
	di in gr `"Case variable: `=abbrev("`e(case)'",24)'"' _col(48) ///
	 "Number of cases" _col(67) "= " in ye %10.0g e(N_case) 
	if ("`e(cmd)'"=="asroprobit" & e(N_ties) > 0)            ///
		di in gr _col(48) "Number of ties" _col(67) "= " ///
		in ye %10.0g e(N_ties) 
	di
	local altvar `=abbrev("`e(altvar)'",17)'
	di in gr `"Alternative variable: `altvar'"' _col(48)               ///
	 "Alts per case: min = " in ye %10.0g e(alt_min) _n _col(63) in gr ///
	 "avg = " in ye %10.1f e(alt_avg) _n _col(63) in gr "max = "       ///
	 in ye %10.0g e(alt_max)
	local lencr = length("`e(crittype)'")
	if ("`e(mc_method)'" != "") {
/****************************************************************************/
local sk = `lencr'-length("`e(mc_method)'") - 8
di in smcl in gr "Integration sequence:" _skip(`sk') "`e(mc_method)'" 
local sk = `lencr'-21
local intpoints ///
	`""Integration points:" _skip(`sk') in ye %15.0g `=e(mc_points)'"'
/****************************************************************************/
	}
	else local intpoints _n

	if "`e(chi2type)'" == "Wald" {
		local stat chi2
        	local cfmt=cond(e(chi2)<1e+7,"%10.2f","%10.3e")
       		if e(chi2) >= . {
			local h help j_robustsingular:
			di in gr `intpoints' in gr _col(51)    ///
			 "{`h'Wald chi2(`e(df_m)'){col 67}= }" ///
			 in ye `cfmt' e(chi2)
       		}
        	else {
			di in gr `intpoints' in gr _col(51)     	   ///
			 "`e(chi2type)' chi2(" in ye "`e(df_m)'" in gr ")" ///
			 _col(67) "= " in ye `cfmt' e(chi2)
       		}
	}
	else {
		/* F statistic from test after _robust */
		local stat F
		local cfmt=cond(e(F)<1e+7,"%10.2f","%10.3e")
		if e(F) < . {
			di in gr `intpoints' in gr _col(51) 		///
			 "F(" in ye %3.0f e(df_m) in gr "," in ye %6.0f ///
			 e(df_r) in gr ")" _col(67) "= " in ye `cfmt' e(F)
		}
		else {
			local dfm = e(df_m)
			local dfr = e(df_r)
			local h help j_robustsingular:

			di in gr `intpoints' in gr _col(51) ///
			 "{`h'F( `dfm', `dfr')}" _col(67) "{`h'=          .}"
		}
	}
	di in gr "`crtype' = " in ye %10.0g e(ll) _col(51) in gr ///
	 "Prob > `stat'" _col(67) "= "  in ye %10.4f e(p) _n

	local coleq : coleq e(b)
	local coleq : list uniq coleq
	_choice_table, begin level(`level')
	if e(k_indvars) {
		if e(const)+e(k_casevars)+`ncov' == 0 {
			_choice_table `e(altvar)', equation level(`level') ///
				nosep
		}
		else {
			_choice_table `e(altvar)', equation level(`level') 
		}
	}
	if e(const) + e(k_casevars) > 0 {
		forvalues i=1/`=e(k_alt)' {
			if (`i'==`=e(k_alt)' & `ncov'==0) local nosep nosep
			local altid : word `i' of `e(alteqs)'
			if `i' == e(i_base) {
				_choice_table `altid', 			///
					comment("  (base alternative)") ///
					`nosep'
			}
			else {
				_choice_table `altid', equation ///
					level(`level') `nosep'
			}
		}
	}

	if `ncov' > 0 {
		if e(k_factors) < . {
			Replay2 `level'
		}
		else {
			Replay1 `level'
		}
	}	
	_get_diparmopts, bottom execute
	if e(k_alt) > 2 {
		local i = `e(i_base)'
		if (wordcount("`e(alt`i')'") > 1) local base ///
			`""`=abbrev(`"`e(alt`i')'"',17)'""'
		else local base `"`=abbrev(`"`e(alt`i')'"',17)'"'
        	di in gr `"{p}(`altvar'=`base' is the alternative "' ///
		 `"normalizing location){p_end}"'
		if (`e(i_scale)' < .) {
			local i = `e(i_scale)'
			if (wordcount("`e(alt`i')'") > 1) local scale ///
				`""`=abbrev(`"`e(alt`i')'"',17)'""'
			else local scale `"`=abbrev(`"`e(alt`i')'"',17)'"'
		        di in gr `"{p}(`altvar'=`scale' is the "' ///
			 `"alternative normalizing scale){p_end}"'
		}
	}
	ml_footnote
end

program Replay1
	args level

	local nalt = e(k_alt)
	if `e(cholesky)' {
		local rii lnl
		local labii l
		local rij l
	}
	else {
		local rii lnsigma
		local labii sigma
		local rij atanhr
	}
	if e(k_sigma) > 0 {
		if !e(fullcov) {
			forvalues i = 1/`e(k_sigma)' {
				_diparm lnsigmaP`i', level(`level')
			}
		}
		else {
			forvalues i = 1/`=colsof(e(stdpattern))' {
				if (el(e(stdpattern),1,`i') >= .) continue
	
				if e(cholesky) local ii `i'_`i'
				else local ii `i'
	
				_diparm `rii'`ii', level(`level')
			}
		}
		if (e(k_rho)>0 | `e(cholesky)'==0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
	}
	if e(k_rho) > 0 {
		if !e(fullcov) {
			forvalues i=1/`e(k_rho)' {
				_diparm atanhrP`i', level(`level')
			}
		}
		else {
			forvalues j=1/`=rowsof(e(corpattern))' {
				local j1 = `j'+1
				forvalues i=`j1'/`=rowsof(e(corpattern))' {
					if e(fullcov) local ij `i'_`j'
					else local ij = ///
						el(e(corpattern),`i',j')
					if (el(e(corpattern),`i',`j') >= .) ///
						continue
					_diparm `rij'`ij', level(`level')
				}
			}
		}
		if (`e(cholesky)'==0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
	}
	if (e(cholesky) != 0) exit
	if !e(fullcov) local P P
	if e(k_sigma) > 0 {
		forvalues i = 1/`nalt' {
			if (el(e(stdpattern),1,`i') >= .) {
				if (`i'==e(i_base)) local cmnt ///
					"(base alternative)"
				else if (`i'==e(i_scale)) local cmnt ///
					"(scale alternative)"
				else local cmnt "(fixed)"

				_choice_table sigma`i', ///
					value(`=el(e(stdfixed),1,`i')') ///
					comment("  `cmnt'") nosep
			}
			else {
				if e(fullcov) local ii `i'
				else local ii = el(e(stdpattern),1,`i')
	
				_diparm lnsigma`P'`ii', ///
				level(`level') exp label(sigma`i')
			}
		}
	}
	if e(k_rho) > 0 {
		if (e(k_sigma) > 0) ///
			_get_diparmopts, diparmopts(diparm(__sep__)) execute
		forvalues j=1/`nalt' {
			local j1 = `j'+1
			forvalues i=`j1'/`nalt' {
				if e(fullcov) local ij `i'_`j'
				else local ij = el(e(corpattern),`i',`j')
/****************************************************************************/
		if (el(e(corpattern),`i',`j') >= .) {
			if `i'== e(i_base)|`j' == e(i_base) {
				if (el(e(corfixed),`i',`j')==0) continue
				else local cmnt "(base alternative)"
			}
		    	else local cmnt "(fixed)"

			_choice_table rho`i'_`j', 		  ///
				value(`=el(e(corfixed),`i',`j')') ///
				comment("  `cmnt'") nosep
		}
		else {
			_diparm atanhr`P'`ij', level(`level') ///
				function(tanh(@)) 	      ///
				derivative((1-tanh(@)^2))     ///
						label(rho`i'_`j')
		}
/****************************************************************************/
			}
		}
	}
end

program Replay2
	args level

	local dim = e(k_alt)
	if !e(structcov) {
		local `--dim'
	}
	local nfac = e(k_factors)
	if (`nfac' >= .) exit

	tempname fixed
	mat `fixed' = e(stdfixed)
	forvalues j=1/`dim' {
		if `fixed'[1,`j'] >=. {
			forvalues i=1/`nfac' {
				_diparm c`i'_`j', level(`level')
			}
		}
	}
end

exit
