*! version 1.1.0  01apr2008
program _mixed_parsecmd
	version 9
	gettoken i 0 : 0

	// First we strip off if and in - they have already been processed
	syntax [anything] [if] [in] [, * ]
	local 0 `anything', `options'

	local myopts noConstant COVariance(name) COLlinear

	gettoken lev rest : 0 , parse(":")

if 0$XTM_ver > 10 {		// As of 10.1, the colon is required
	gettoken colon rest : rest , parse(":")
	if `"`colon'"' == `":"' {
		if "`lev'" != "_all" {
			unab lev : `lev'
			confirm variable `lev'
		}
		local 0 `"`rest'"'
	}
	else {
		if (`i' > 0)  {
			di as err "{p 0 6 2}invalid random-effects "
			di as err "specification; perhaps you omitted the "
			di as err "colon after the level "
			di as err "variable{p_end}"
			exit 198
		}
		else	      local lev
	}
}

else {				// old syntax allowing empty level variable
	if `"`lev'"' == `":"' {
		local lev _all
		local 0 `"`rest'"'
	}
	else {
		gettoken colon rest : rest , parse(":")
		if `"`colon'"' == `":"' {
			if (`"`lev'"' == `"."') {
				local lev _all
			}
			else if "`lev'" != "_all" {
				unab lev : `lev'
				confirm variable `lev'
			}
			local 0 `"`rest'"'
		}
		else {
			if (`i' > 0)  local lev _all
			else	      local lev
		}
	}
}

	capture syntax [varlist(ts numeric default=none)]		///
		       [ , `myopts' ]

	if _rc {						// r.varname...
	    if `i' > 0 {
		ChkNonNumeric `"`myopts'"' `0'

		syntax [anything] [ , `myopts' ]

		if 0`:list sizeof anything' != 1 {
		    di as error `"`anything' invalid level specification"'
		    exit 198
		}

		local anything0 `"`anything'"'
		local anything : subinstr local anything "*" " " , all
		if 0`:list sizeof anything' == 1 {		// r.varname
		    tokenize `"`anything'"' , parse(".")
		    if `"`=upper(`"`1'"')'"' != `"R"' | `"`4'"' != `""' {
			di as error `"`anything0' invalid random factor"'
			exit 198
		    }
		    confirm variable `3'
		    local varlist `3'
		}
		else {						// r.v1*r.v2...
		    gettoken fvar anything : anything
		    while "`fvar'" != "" {
			tokenize `"`fvar'"' , parse(".")
			if `"`=upper(`"`1'"')'"' != `"R"' | `"`4'"' != `""' {
			    di as error `"`anything0' invalid random factor"'
			    exit 198
			}
			confirm variable `3'
			local varlist "`varlist' `3'"

			gettoken fvar anything : anything
			local anything `anything'		// sic
		    }
		}
		tsunab varlist : `varlist'
	    }
	    else {
	    	syntax [varlist(ts numeric default=none)]		///
		       [ , `myopts' ]
	    }

	    local  isfctr 1
	}
	else local isfctr 0

	ParseCov cov : `isfctr' , `covariance'

	if !0`isfctr' & "`cov'" != "identity" &				///
	     `:list sizeof varlist' + ("`constant'"=="") == 1 {
	    if `"`covariance'"' != `""' {
		SingleLevelCovNote `lev' `varlist'
	    }
	    local cov identity
	}

	if `i' == 0  {					// parsing fixed level
	    if `"`lev'"' != "" {
		di as err "{p 0 4 2}level specification not allowed "
		di as err "for fixed equation{p_end}"
		exit 198
	    }
	    gettoken depname varlist : varlist
	    c_local depname `depname'

	    local cov identity

	    if "`covariance'" != "" {
		di as error						///
		"option covariance() not allowed with fixed-effects equation"
		exit 198
	    }
	    if "`collinear'" != "" {
	    	di as error						///
		"option collinear not allowed with fixed-effects equation"
		exit 198
	    }
	}

	c_local levnm_`i' `lev'
	c_local cov_`i'      `cov'
	c_local isfctr_`i'   `isfctr'
	c_local constant_`i' = cond(`isfctr', "noconstant", "`constant'")
	c_local collin_`i' "`collinear'"
	c_local varnames_`i'  `varlist'
end

program ParseCov
	gettoken lmac   0 : 0
	gettoken colon  0 : 0
	gettoken isfctr 0 : 0


	syntax [ , INDependent EXchangeable IDentity UNstructured * ]

	if `"`options'"' != "" {
		di as error `"`options' invalid covariance structure"'
		exit 198
	}

	local res `independent' `exchangeable' `identity' `unstructured'

	if 0`isfctr' & "`independent'`unstructured'" != "" {
	di as error							///
"`independent'`unstructured' covariance structure not allowed for factor variables"
		exit 198
	}

	if ("`res'" == "") {
		if (0`isfctr')	local res "identity"
		else		local res "independent"
	}

	c_local `lmac' `res'
end

program ChkNonNumeric
	gettoken myopts 0 : 0

	local 0 : subinstr local 0 "R." "" ,  all count(local ct)
	if (`ct' > 0)  exit					// Exit

	local 0 : subinstr local 0 "r." "" ,  all count(local ct)
	if (`ct' > 0)  exit					// Exit

	syntax [varlist(ts numeric default=none)] [if] [in] [ , `myopts' ]

end

program SingleLevelCovNote
	gettoken lev varlist : 0

	di as text "{p 0 6} Note: single-variable random-effects specification; covariance structure set to {inp:identity}{p_end}"
end

exit
