*! version 1.2.2  24mar2008
program _robust2, rclass
	version 9
	syntax varlist(numeric) [if] [in]	///
		[pw iw fw aw] [,		///
		SVY				///
		CLuster(varname)		///
		PSU(varname)			/// not documented
		STRata(varname)			///
		FPC(varname numeric)		///
		SUBpop(passthru)		///
		SRSsubpop			/// ignored
		Variance(name)			///
		VSRS(name)			///
		minus(numlist max=1 int >=0)	///
		ZEROweight			///
		TOUSE1(name)			///
		ALLCONS				///
		GROUP(varname)			///
	]

	local cluster `cluster' `psu'
	if `: list sizeof cluster' > 1 {
		opts_exclusive "cluster() psu()"
	}

	if `:length local cluster' {
		local svyopts cluster(`cluster')
	}
	if `:length local strata' {
		local svyopts `svyopts' strata(`strata')
	}

	if "`minus'" == "" {
		local minus 1
	}
	if "`svy'" != "" & "`weight'" != "" {
		di as err ///
"weights can only be supplied to {help svyset##|_new:svyset}"
		exit 198
	}
	if "`svy'" != "" | "`weight'" == "iweight" {
		local zeroweight zeroweight
	}
	marksample touse, `zeroweight'

	if "`variance'" != "" {
		confirm matrix `variance'
	}
	else {
		if "`e(V)'" != "matrix" {
			error 301
		}
		local vartmp yes
		tempname variance
		matrix `variance' = e(V)
	}

	local k_scores : list sizeof varlist
	tempname eqdims
	// verify # vars in varlist == # equations is `variance'
	CheckMatrixLabels `k_scores' `variance' `eqdims' `allcons'
	local xvarlist	`"`r(varlist)'"'
	local eqnames	`"`r(eqnames)'"'
	local namelist	`"`r(namelist)'"'
	local has_ts	= r(has_ts)
	local xlist : subinstr local namelist "_cons" "`touse'", all word
	markout `touse' `fpc'

	if `has_ts' {
		local ulist0 : list uniq xlist
		tsrevar `ulist0'
		local ulist1 "`r(varlist)'"
		while `:list sizeof ulist1' {
			gettoken u0 ulist0 : ulist0
			gettoken u1 ulist1 : ulist1
			if !`:list u0 == u1' {
				local xlist : subinstr local xlist	///
					"`u0'" "`u1'", all word
			}
		}
	}

	if "`strata'`cluster'" != "" {
		markout `touse' `strata' `cluster', strok
	}

	if ! `:length local svy' {
		local altsvy = "`strata'`cluster'`fpc'`subpop'" != ""
	}
	else	local altsvy 0

	// get the -svyset- information
	tempvar subuse
	_svy_setup `touse' `subuse', `svy' `svyopts' `subpop' `srssubpop'
	return add

	local nobs1 = c(N) - return(N) + 1
	if `nobs1' != 1 {
		local ifin if `touse'
	}
	local i1i2 "."

	return local settings
	return local brrweight
	return local jkrweight
	if "`svy'" != "" {
		local stages	= return(stages)
		if missing(`stages') {
			local stages 1
		}
		local hasstr 0
		forval i = 1/`stages' {
			if "`return(strata`i')'" != "" {
				local stri `return(strata`i')'
				if substr("`:type `stri''",1,3) == "str" {
					tempvar stru
					qui egen `stru' = group(`stri') `ifin'
					local stri `stru'
				}
				local strata `strata' `stri'
				local sortlist `sortlist' `stri'
				local ++hasstr
			}
			else {
				// NOTE: `sui' exists because you
				// cannot have stratification at level
				// i if the sampling units are the
				// observations at level i-1
				if `i' == 1 & `stages' > 1 {
					local strata `touse'
				}
				else if `i' < `stages' {
					local strata `strata' `sui'
				}
			}
			if "`return(su`i')'" != "" {
				local sui `return(su`i')'
				if substr("`:type `sui''",1,3) == "str" {
					tempvar sutmp
					qui egen `sutmp' = group(`sui') `ifin'
					local sui `sutmp'
				}
				local su `su' `sui'
				local sortlist `sortlist' `sui'
			}
			// else there is one less su var than stages
			local fpc	`fpc' `return(fpc`i')'
		}
		if !`hasstr' {
			local strata
		}
		local posts	`return(poststrata)'
		local postw	`return(postweight)'
		if `:length local posts' {
			local postopts posts(`posts') postw(`postw')
		}
		local singleu	`return(singleunit)'
		local wtype	`return(wtype)'
		local wexp	`"`return(wexp)'"'
		if "`wexp'" != "" {
			tempvar wvar
			quietly gen double `wvar' `wexp'
		}
		local vce linearized
	}
	else {
		if "`cluster'" != "" {
			if substr("`:type `cluster''",1,3) == "str" {
				tempvar clvar
				qui egen `clvar' = group(`cluster') `ifin'
			}
			else	local clvar `cluster'
		}
		if "`strata'" != "" {
			if substr("`:type `strata''",1,3) == "str" {
				tempvar strvar
				qui egen `strvar' = group(`strata') `ifin'
			}
			else	local strvar `strata'
		}
		if "`weight'" != "" {
			tempvar wvar
			quietly gen double `wvar' `exp' `ifin'
			local wexp `"`:list retok exp'"'
		}
		local stages	1
		local su	`clvar'
		local strata	`strvar'
		local sortlist	`strata' `su'
		local wtype	`weight'
	}

	// poststratification
	if "`posts'" != "" {
		tempvar postid
		GenId `postid' `touse' `posts' `nobs1'
		local postvars `postid' `postw'

		// do not return vsrs() for poststratification
		local vsrs
	}

	local mvarlist	`varlist'	///
			`xlist'		///
			`subuse'	///
			`wvar'		///
			`fpc'		///
			`strata'	///
			`su'		///
			`postvars'
	local mvarlist	: list retok mvarlist
	local kscores	: list sizeof varlist
	local kxlist	: list sizeof xlist
	local ksubuse	: list sizeof subuse
	local kwvar	: list sizeof wvar
	local kfpc	: list sizeof fpc
	local kstrata	: list sizeof strata
	local ksu	: list sizeof su
	local kpostvars	: list sizeof postvars

	tempname v
	matrix `v' = `variance'
	if `"`sortlist'"' != "" {
		if `nobs1' != 1 {
			local i1i2 "(`nobs1', `c(N)')"
		}
		local SORTP "SortP `touse' `sortlist' : `mvarlist' :"
	}
	capture `SORTP'		///
	mata: _robust2(		///
		"`v'",		/// OUTPUT: variance estimates
		"`vsrs'",	/// OUTPUT: SRS variance estimates
		`i1i2',		/// INPUT:  (i1, i2)
		"mvarlist",	/// INPUT:  local holding stata variables
		"`touse'",	/// INPUT:  touse variable
		`kscores',	/// INPUT:  # of score vars
		`kxlist',	/// INPUT:  # of x vars
		"`eqdims'",	/// INPUT:  stata matrix of equation dims
		`ksubuse',	/// INPUT:  .,0,1 for subuse var
		`kwvar',	/// INPUT:  .,0,1 for wvar
		`stages',	/// INPUT:  # of stages
		`kfpc',		/// INPUT:  # of fpc vars
		`kstrata',	/// INPUT:  # of strata vars
		`ksu',		/// INPUT:  # of sample unit vars
		`kpostvars',	/// INPUT:  .,0,2 for poststrata vars
		"`singleu'",	/// INPUT:  single unit string code
		`minus',	/// INPUT:  from minus() option
		"`wtype'"	/// INPUT:  weight type
	)

	if c(rc) error c(rc)

	// saved results
	matrix `variance'[1,1] = `v'
	if _caller() < 9 {
		if "`vsrs'" != "" {
			if "`srssubpop'" != "" {
				matrix `vsrs' = `vsrs'sub
				capture matrix drop `vsrs'sub
			}
			capture matrix drop `vsrs'wr
			capture matrix drop `vsrs'subwr
			local Vlist `vsrs'
		}
	}
	else {
		if "`vsrs'" != "" {
			local Vlist `vsrs' `vsrs'sub
			if "`vsrs'" != "" & "`fpc'" != "" {
				local Vlist `Vlist' `vsrs'wr `vsrs'subwr
			}
		}
	}
	return scalar census = r(census)
	foreach mat of local Vlist {
		matrix colna `mat' = `xvarlist'
		matrix rowna `mat' = `xvarlist'
		matrix coleq `mat' = `eqnames'
		matrix roweq `mat' = `eqnames'
	}
	local sclist : r(scalars)
	foreach sc of local sclist {
		return scalar `sc' = r(`sc')
	}
	local matlist : r(matrices)
	tempname m
	foreach mat of local matlist {
		matrix `m' = r(`mat')
		return matrix `mat' `m'
	}
	if "`svy'`group'" != "" | `altsvy' {
		return scalar df_r = r(N_clust) - r(N_strata)
		if `"`subpop'"' == "" {
			return local N_sub
			return local sum_wsub
		}
		if `:length local group' {
			if !`:length local wvar' {
				local wvar `touse'
			}
			if `:length local subpop' {
				local subopt subuse(`subuse')
			}
			GroupPop `wvar',	///
				group(`group')	///
				touse(`touse')	///
				`subopt'	///
				`postopts'
			return scalar N_pop = r(N_pop)
			if `:length local subpop' {
				return scalar N_subpop = r(N_subpop)
			}
		}
	}
	else {
		return local N_sub
		return local sum_wsub
		if `"`subpop'"' == "" {
			return local N_sub
			return local sum_wsub
		}
		return local _N_strata
		return local _N_strata_single
		return local _N_strata_certain
		if "`wtype'" == "fweight" {
			return scalar N = r(sum_w)
			if "`cluster'" == "" {
				return scalar N_clust = r(sum_w)
			}
			return scalar df_r = r(sum_w) - 1
		}
		else	return scalar df_r = r(N) - 1
		if "`wtype'" != "" {
			return local wtype `wtype'
			return local wexp  `"`wexp'"'
		}
	}
	if "`touse1'" != "" {
		capture confirm new var `touse1'
		if c(rc) {
			quietly replace `touse1' = `touse'
		}
		else	rename `touse' `touse1'
	}
	if "`vartmp'" == "yes" {
		Repost `variance' "`cluster'"
	}
end

program GroupPop, rclass sortpreserve
	syntax [varname],			///
		group(varname)			///
		touse(varname)			///
		[				///
			subuse(varname)		///
			POSTStrata(passthru)	///
			POSTWeight(passthru)	///
		]

	if `:length local poststrata' {
		tempvar w
		if `:length local varlist' {
			local wgt [pw=`varlist']
		}
		svygen post double `w' `wgt' if `touse',	///
			`poststrata' `postweight'
	}
	else	local w `varlist'

	sort `group' `touse' `subuse'
	if `:length local w' {
		sum `w' if `touse' & `group' != `group'[_n+1], meanonly
		return scalar N_pop = r(sum)
		if `:length local subuse' {
			sum `w' if `subuse' & `group' != `group'[_n+1], mean
			return scalar N_subpop = r(sum)
		}
	}
	else {
		count if `touse' & `group' != `group'[_n+1]
		return scalar N_pop = r(N)
		if `:length local subuse' {
			count if `subuse' & `group' != `group'[_n+1]
			return scalar N_subpop = r(N)
		}
	}
end

program SortP, sort
	gettoken sortlist 0 : 0, parse(":")
	gettoken COLON 0 : 0, parse(":")
	gettoken mvarlist 0 : 0, parse(":")
	gettoken COLON 0 : 0, parse(":")
	sort `sortlist'
	`0'
end

program Repost, eclass
	args v cluster
	ereturn repost V = `v'
	ereturn local vcetype "Robust"
	if "`cluster'" != "" {
		ereturn local vce cluster
		ereturn local clustvar "`cluster'"
		ereturn scalar N_clust = r(N_clust)
	}
	else	ereturn local vce robust
end

program CheckMatrixLabels, rclass
	args k v eqdims allcons
	local cons _cons
	local eqnames	: coleq `v', quote
	local ueq	: list uniq eqnames
	local names	: colna `v'

	if "`allcons'" == "" {
		local k_eq : list sizeof ueq
	}
	else {
		local k_eq : list sizeof names
	}
	if `k' < `k_eq' {
		error 102
	}
	else if `k' > `k_eq' {
		error 103
	}
	local vlist : subinstr local names "_cons" "", all word
	local vlist : list uniq vlist
	if "`vlist'" != "" {
		ConfirmVars `vlist'
		local junk : subinstr local vlist "." "", count(local has_ts)
	}
	else	local has_ts 0

	matrix `eqdims' = J(1,`k',0)
	if "`allcons'" == "" {
		forval i = 1/`k' {
			local eq : word `i' of `ueq'
			local ignored :					///
				subinstr local eqnames `""`eq'""' "",	///
				all word count(local cnt)
			if `i' == 1 {
				matrix `eqdims'[1,`i'] = `cnt'
			}
			else {
				matrix `eqdims'[1,`i'] = `eqdims'[1,`i'-1]+`cnt'
			}
		}
	}
	else {
		forval i = 1/`k' {
			matrix `eqdims'[1,`i'] = `i'
			local conslist `conslist' _cons
		}
	}
	return local has_ts = `has_ts'
	return local varlist	`"`names'"'
	_mat_clean_coleq eqnames : `eqnames'
	return local eqnames	`"`eqnames'"'
	if "`allcons'" != "" {
		return local namelist `"`conslist'"'
	}
	else	return local namelist `"`names'"'
end

program ConfirmVars
	syntax [varlist(ts numeric default=none)]
end

program GenId, sort
	// Assumption:  the data is sorted by a "touse" variable and the
	// "oldid" variable.
	//
	// Note: this routine is much faster than using -egen, group()- or
	// -by: generate- with an -if-, given the above assumption.
	args newid touse oldid nobs1
	sort `touse' `oldid'
	local in `nobs1'/l
	quietly gen `newid' = `oldid' != `oldid'[_n-1] in `in'
	quietly replace `newid' = 1 in `nobs1'
	quietly replace `newid' = sum(`newid') in `in'
end

exit
