{smcl}
{* *! version 1.0.9  10jun2008}{...}
{cmd:help generate}, {cmd:help replace},{right:dialogs:  {bf:{dialog generate}}  {bf:{dialog replace}}}
{cmd:help set seed}, {cmd:help set type}
{hline}

{title:Title}

{p2colset 5 21 23 2}{...}
{p2col :{hi:[D] generate} {hline 2}}Create or change contents of variable{p_end}
{p2colreset}{...}


{title:Syntax}

{phang}
Create new variable

{p 8 17 2}
{opt g:enerate}
{dtype}
{newvar}[{cmd::}{it:lblname}] {cmd:=}{it:exp}
{ifin}


{phang}
Replace contents of existing variable

{p 8 17 2}
{cmd:replace}
{it:oldvar}
{cmd:=}{it:exp}
{ifin}
[{cmd:,} {opt nop:romote}]


{phang}
Specify initial value of random-number seed

{p 8 17 2}
{marker seed}
{cmd:set}
{opt se:ed}
{c -(}{it:#}{c |}{it:code}{c )-}


{phang}
Specify default storage type assigned to new variables

{p 8 17 2}
{cmd:set}
{opt ty:pe} {c -(}{opt float}{c |}{opt double}{c )-}
[{cmd:,} {opt perm:anently}]


{phang}
where {it:type} is one of{break}
	{opt byte}{c |}{opt int}{c |}{opt long}{c |}{opt float}{c |}{opt double}{c |}
	{opt str}{c |}{opt str1}{c |}{opt str2}{c |}...{c |}{cmd:str{ccl maxstrvarlen}}

{phang}
See Description below for an explanation of {opt str}.  For the other
types, see {manhelp data_types D:data types}.

{phang}
{opt by} is allowed with {opt generate} and {opt replace}; see {manhelp by D}.


{title:Description}

{pstd}
{opt generate} creates a new variable.  The values of the variable are
specified by {cmd:=}{it:exp}.

{pstd}
If no {it:type} is specified, the new variable type is determined by the type
of result returned by {cmd:=}{it:exp}.  A {opt float} variable (or a
{opt double}, according to {opt set type}) is created if the result is numeric,
and a string variable is created if the result is a string.  In the latter
case, a {opt str}{it:#} variable is created where {it:#} is the smallest
string that will hold the result.

{pstd}
If a {it:type} is specified, the result returned by {cmd:=}{it:exp} must
be string or numeric according to whether {it:type} is string or numeric.  If
{opt str} is specified, a {opt str}{it:#} variable is created, where {it:#} is
the smallest string that will hold the result.

{pstd}
See {manhelp egen D} for extensions to {opt generate}.

{pstd}
{opt replace} changes the contents of an existing variable.
Because {opt replace} alters data, the command cannot be abbreviated.

{pstd}
{opt set seed} {it:#} specifies the initial value of the random-number seed
used by the {helpb random:runiform()} function.  {it:#} should be a positive
integer. You may also specify {it:code} to indicate a previous internal state.

{pstd}
{opt set seed} {it:code} specifies a string that records the state of the
random-number generator {cmd:runiform()}.  You can obtain the current state of
the random-number generator from {cmd:c(seed)}.

{pstd}
{opt set type} specifies the default storage type assigned to new variables
(such as those created by {opt generate}) when the storage type is not
explicitly specified.


{title:Options}

{phang}
{opt nopromote} prevents {opt replace} from promoting the 
{help data types:variable type} to accommodate the change.  For instance,
consider a variable stored as an integer type ({opt byte}, {opt int}, or
{opt long}), and assume that you {opt replace} some values with nonintegers.
By default, {opt replace} changes the variable type to a floating point
({opt float} or {opt double}) and thus correctly stores the changed
values.  Similarly, {opt replace} promotes {opt byte} and {opt int}
variables to longer integers ({opt int} and {opt long}) if the replacement
value is an integer but is too large in absolute value for the current storage
type.  {opt replace} promotes strings to longer strings.  {opt nopromote}
prevents {opt replace} from doing this; instead, the replacement values are
truncated to fit the current storage type.

{phang}
{opt permanently} specifies that, in addition to making the change right now,
the new limit be remembered and become the default setting when you invoke
Stata.


{title:Examples}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse genxmpl3}

{pstd}Create new variable {cmd:age2} containing the values of {cmd:age}
squared{p_end}
{phang2}{cmd:. generate age2 = age^2}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:webuse genxmpl3, clear}

{pstd}Create variable {cmd:age2} with a storage type of {cmd:int} and
containing the values of {cmd:age} squared{p_end}
{phang2}{cmd:. generate int age2 = age^2}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse genxmpl1, clear}

{pstd}Replace the values in {cmd:age2} with those of {cmd:age^2}{p_end}
{phang2}{cmd:. replace age2 = age^2}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse genxmpl2, clear}

{pstd}List the {cmd:name} variable{p_end}
{phang2}{cmd:. list name}

{pstd}Create variable {cmd:lastname} containing the second word of {cmd:name}
{p_end}
{phang2}{cmd:. generate lastname = word(name,2)}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse genxmpl3, clear}

{pstd}Create variable {cmd:age2} with a storage type of {cmd:int} and
containing the values of {cmd:age} squared for all observations for which
{cmd:age} is more than 30{p_end}
{phang2}{cmd:. generate int age2 = age^2 if age > 30}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse genxmpl4, clear}

{pstd}Replace the value of {cmd:odd} in the third observation{p_end}
{phang2}{cmd:. replace odd = 5 in 3}{p_end}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. webuse stan2, clear}

{pstd}Create duplicate of every observation for which {cmd:transplant} is true
(!=0){p_end}
{phang2}{cmd:. expand 2 if transplant}{p_end}

{pstd}Sort observations into ascending order of {cmd:id}{p_end}
{phang2}{cmd:. sort id}

{pstd}Create variable {cmd:posttran}, with storage type of {cmd:byte}, equal
to 1 for the second observation of each {cmd:id} and equal to 0
otherwise{p_end}
{phang2}{cmd:. by id: generate byte posttran = (_n==2)}

{pstd}Create variable {cmd:t1} equal to {cmd:stime} for the last observation
of {cmd:id}{p_end}
{phang2}{cmd:. by id: generate t1 = stime if _n==_N}

    {hline}
{pstd}Setup{p_end}
{phang2}{cmd:. clear all}

{pstd}Specify initial value of random-number seed{p_end}
{phang2}{cmd:. set seed 339487731}

{pstd}Create variable {cmd:random} containing uniformly distributed
pseudorandom numbers on the interval [0,1){p_end}
{phang2}{cmd:. generate random = runiform()}

{pstd}Create variable {cmd:normal} containing normally distributed random
numbers with mean 0 and standard deviation 1{p_end}
{phang2}{cmd:. generate normal = rnormal()}{p_end}
    {hline}


{title:Also see}

{psee}
Manual:  {bf:[D] generate}

{psee}
Online:  
{manhelp compress D},
{manhelp corr2data D},
{manhelp drawnorm D},
{manhelp edit D},
{manhelp egen D},
{manhelp encode D},
{manhelp functions D},
{manhelp label D},
{manhelp recode D},
{manhelp rename D},{break}
{manhelp exp U:13 Functions and expressions (expressions)},{break}
{manhelp operators U:13 Functions and expressions (operators)}
{p_end}
