*! version 1.0.2  22aug2007
version 10

findfile optimize_include.mata
quietly include `"`r(fn)'"'

mata:

real scalar optimize_result_converged(`OptStruct' S)
{
	return(S.converged)
}

`Errcode' optimize_result_errorcode(`OptStruct' S)
{
	return(S.errorcode)
}

string scalar optimize_result_errortext(`OptStruct' S)
{
	return(S.errortext)
}

`Errcode' optimize_result_returncode(`OptStruct' S)
{
	if (S.errorcode) {
		if (S.errorcode == `Errcode_notfeasible') {
			return(1400)
		}
		if (S.errorcode == `Errcode_badconstr') {
			return(412)
		}
		return(430)
	}
	return(0)
}

real rowvector optimize_result_gradient(`OptStruct' S)
{
	if (S.minimize) {
		return(-S.gradient)
	}
	else	return(S.gradient)
}

real matrix optimize_result_Hessian(`OptStruct' S)
{
	`Errcode'	ec

	if (S.oerrorcode == 0) {
		if ((S.valid) & (S.hessian == `OPT_hessian_default')) {
			if (S.hasCns) {
				opt__cns_on(S, 2)
			}
			// get the NR Hessian matrix
			S.eval = findexternal(
				sprintf("opt__eval_nr_%s()",
					optimize_init_evaluatortype(S)))
			ec = opt__eval(S,2)
			if (S.hasCns) {
				opt__cns_off(S, 2)
			}
			if (ec) return(J(S.dim,S.dim,.))
			S.hessian	= S.H
		}
		if (S.minimize) {
			return(-S.hessian)
		}
		else	return(S.hessian)
	}
	return(J(S.dim,S.dim,.))
}

real colvector optimize_result_iterationlog(`OptStruct' S)
{
	real colvector log
	if (S.iter < 20) {
		log = S.log[|1 \ S.iter|]
	}
	else {
		real scalar i, j
		log = `OPT_log_default'
		j = mod(S.iter,20)
		for (i=1; i<=20; i++) {
			if (j == 20) {
				j = 1
			}
			else	j++
			log[i] = S.log[j]
		}
	}
	return(log)
}

real scalar optimize_result_iterations(`OptStruct' S)
{
	return(S.iter)
}

function optimize_result_params(`OptStruct' S)
{
	return(S.params)
}

function optimize_result_scores(`OptStruct' S)
{
	if (S.evaltype < `OPT_evaltype_v0'
	 |  S.evaltype > `OPT_evaltype_max') {
		_error(sprintf(
		"optimize_result_scores() is not allowed with %s evaluators",
			optimize_init_evaluatortype(S)))
	}
	return(S.gradient_v)
}

real scalar optimize_result_value0(`OptStruct' S)
{
	if (S.minimize) {
		return(-S.v0)
	}
	else	return(S.v0)
}

real scalar optimize_result_value(`OptStruct' S)
{
	if (S.minimize) {
		return(-S.value)
	}
	else	return(S.value)
}

real matrix optimize_result_V(`OptStruct' S)
{
	if (S.technique == "bhhh") {
		return(optimize_result_V_opg(S))
	}
	return(optimize_result_V_oim(S))
}

string scalar optimize_result_Vtype(`OptStruct' S)
{
	if (S.technique == "bhhh") {
		return("opg")
	}
	return("oim")
}

function optimize_result_V_oim(`OptStruct' S)
{
	if (S.hasCns) {
		real matrix H
		H = S.T'*optimize_result_Hessian(S)*S.T
		if (S.minimize) {
			H = S.T*invsym(H)*S.T'
		}
		else {
			H = S.T*invsym(-H)*S.T'
		}
		return(H)
	}
	else {
		if (S.minimize) {
			return(invsym(optimize_result_Hessian(S)))
		}
		else	return(invsym(-optimize_result_Hessian(S)))
	}
}

function optimize_result_V_opg(`OptStruct' S)
{
	if (S.evaltype < `OPT_evaltype_v0'
	 |  S.evaltype > `OPT_evaltype_max') {
		_error(sprintf(
		"optimize_result_V_opg() is not allowed with %s evaluators",
			optimize_init_evaluatortype(S)))
	}

	real matrix	H
	if (S.wtype == `OPT_wtype_none') {
		H	= quadcross(
				S.gradient_v,
				S.gradient_v)
	}
	else if (S.wtype == `OPT_wtype_f') {
		H	= quadcross(
				S.gradient_v,
				S.weights,
				S.gradient_v)
	}
	else {
		H	= quadcross(
				S.gradient_v,
				S.weights :* S.weights,
				S.gradient_v)
	}

	if (S.hasCns) {
		H = S.T'*H*S.T
		H = invsym(H)
		H = S.T*H*S.T'
		return(H)
	}
	else	return(invsym(H))
}

function optimize_result_V_robust(`OptStruct' S)
{
	if (S.evaltype < `OPT_evaltype_v0'
	 |  S.evaltype > `OPT_evaltype_max') {
		_error(sprintf(
		"optimize_result_V_robust() is not allowed with %s evaluators",
			optimize_init_evaluatortype(S)))
	}

	real scalar	nobs
	real matrix	H

	H	= optimize_result_V_oim(S)
	if (S.wtype == `OPT_wtype_none') {
		nobs	= rows(S.gradient_v)
		H	= (nobs/(nobs-1)) *
		  	H * (quadcross(S.gradient_v,
				 	S.gradient_v)) * H'
	}
	else if (S.wtype == `OPT_wtype_f') {
		nobs	= quadcolsum(S.weights)
		H	= (nobs/(nobs-1)) *
		  	H * (quadcross(S.gradient_v,S.weights,
				 	S.gradient_v)) * H'
	}
	else {
		nobs	= rows(S.gradient_v)
		H	= (nobs/(nobs-1)) *
		  	H * (quadcross(S.gradient_v,
					S.weights :* S.weights,
				 	S.gradient_v)) * H'
	}

	_makesymmetric(H)
	return(H)
}

end
