*! version 1.0.6  01may2008
version 10

findfile optimize_include.mata
quietly include `"`r(fn)'"'

mata:

/*STATIC*/ `Errcode' opt__bfgs(`OptStruct' S)
{
	if (S.iter == 0) {
		S.oldparams	= S.params
		S.oldgrad	= S.gradient
		S.H	= I(S.dim)
		return(0)
	}

	real scalar	eps
	real scalar	dbdgp, dgHdgp, gPg, bPb
	real colvector	d
	real matrix	db, dg
	real matrix	V

	V	= S.H
	eps	= 1e-8
	db	= S.params - S.oldparams
	dg	= S.gradient - S.oldgrad
	dbdgp	= db * dg'
	dgHdgp	= dg * V * dg'
	bPb	= db * db'
	gPg	= dg * dg'
	if (abs(dbdgp*dgHdgp) > eps*gPg*bPb) {
		d = db' / dbdgp - V*dg' / dgHdgp
		V = V - db'*db/dbdgp + dgHdgp*d*d' - (V*dg')*(dg*V')/dgHdgp
		V = (V + V')/2
		if (missing(V)) {
			S.errorcode = `Errcode_unstable'
			S.errortext = `Errtext_unstable'
			opt__errorhandler(S)
			return(S.errorcode)
		}
		S.H	= V
	}
	else {
		if (S.reset < 10) {
			if (S.trace > `OPT_trace_none') {
				displayas("txt")
	printf("BFGS stepping has contracted, resetting BFGS Hessian\n")
			}
			S.H	= I(S.dim)
			S.reset	= S.reset + 1
		}
		else {
			S.errorcode = `Errcode_flat'
			S.errortext = `Errtext_flat'
			opt__errorhandler(S)
			return(S.errorcode)
		}
	}

	S.oldparams	= S.params
	S.oldgrad	= S.gradient
	return(0)
}

/*STATIC*/ void opt__bhhh(`OptStruct' S)
{
	// NOTE: 'optimize()' assumes it is getting the Hessian matrix back,
	// but we are computing the outer product of the gradients here, which
	// is an approximation of -H.  We also use deviations from the mean
	// since the gradient is only near the zero vector at the solution.
	if (S.wtype == `OPT_wtype_none') {
		S.H	= -quadcross(
			S.gradient_v,
			S.gradient_v)
	}
	else if (S.wtype == `OPT_wtype_f') {
		S.H	= -quadcross(
			S.gradient_v,
			S.weights,
			S.gradient_v)
	}
	else {
		S.H	= -quadcross(
			S.gradient_v,
			S.weights :* S.weights,
			S.gradient_v)
	}
}

/*STATIC*/ void opt__calluser0_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	pragma unset arg
	(*user)(
		todo,
		p,
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser0_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// INPUT  : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	pragma unset arg
	(*user)(
		todo,
		p,
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser1_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser1_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// INPUT  : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser2_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser2_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser3_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser3_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser4_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser4_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser5_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser5_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser6_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser6_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser7_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser7_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser8_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		(*(arg[8])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser8_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		(*(arg[8])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ void opt__calluser9_d(
	pointer(real function)	scalar		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient vector
	real matrix	H		// OUTPUT : Hessian matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		(*(arg[8])),
		(*(arg[9])),
		v,
		g,
		H)
	if (minimize) {
		v = -v
		if (todo > 0) {
			g = -g
			if (todo > 1) {
				H = -H
			}
		}
	}
}

/*STATIC*/ void opt__calluser9_v(
	pointer(real function)	vector		user,
	pointer(matrix)		rowvector	arg,
	real scalar		minimize,
	real scalar	todo,		// INPUT  : instructions
	real rowvector	p,		// INPUT  : parameter vector
	real colvector	w,		// OUTPUT : weights
	real scalar	v,		// OUTPUT : value
	real rowvector	g,		// OUTPUT : gradient by obs matrix
	real matrix	H,		// OUTPUT : Hessian matrix
	real colvector	v_v,		// OUTPUT : values by obs vector
	real matrix	g_v		// OUTPUT : gradient by obs matrix
)
{
	(*user)(
		todo,
		p,
		(*(arg[1])),
		(*(arg[2])),
		(*(arg[3])),
		(*(arg[4])),
		(*(arg[5])),
		(*(arg[6])),
		(*(arg[7])),
		(*(arg[8])),
		(*(arg[9])),
		v_v,
		g_v,
		H)
	if (minimize) {
		v_v = -v_v
		if (todo > 0) {
			g_v = -g_v
			if (todo > 1) {
				H = -H
			}
		}
	}
	opt__check_v(todo,w,v_v,g_v,v,g)
}

/*STATIC*/ real scalar opt__check_concave(real matrix H, real matrix V)
{
	real vector dH, dV

	dV	= diagonal(V) :== 0
	if (!any(dV)) {
		return(1)
	}
	dH	= diagonal(H) :== 0
	if (!any(dH)) {
		return(0)
	}
	if (dV'(1:-dH) == 0) {
		return(1)
	}
	return(0)
}

/*STATIC*/ void opt__check_v(
	real scalar	todo,		// INPUT  : instructions
	real colvector	w,		// INPUT  : weights
	real colvector	v_v,		// INPUT  : values by obs vector
	real matrix	g_v,		// INPUT  : gradient by obs matrix
	real scalar	v,		// OUTPUT : value
	real rowvector	g		// OUTPUT : gradient by obs matrix
)
{
	if (missing(v_v)) {
		v = .
		return
	}
	else	v = quadcross(w, v_v)
	if (todo > 0) {
		if ( (rows(g_v) > 1) & (rows(v_v) != rows(g_v)) ) {
			if (rows(v_v) < rows(g_v)) {
				_error(3200,
			"evaluator returned gradient vector with too many rows")
			}
			else {
				_error(3200,
			"evaluator returned gradient vector with too few rows")
			}
		}
		if (!missing(g_v)) {
			g = quadcross(w, g_v)
		}
		else	v = .
	}
}

/*STATIC*/ void opt__cns_off(`OptStruct' S, real scalar todo)
{
	S.params = S.params*S.T'+S.a
	if (todo > 0) {
		S.gradient = S.gradient*S.T'
		if (todo > 1) {
			S.H = S.T*S.H*S.T'
		}
	}
}

/*STATIC*/ void opt__cns_on(`OptStruct' S, real scalar todo)
{
	S.params = (S.params-S.a)*S.T
	if (todo > 0) {
		S.gradient = S.gradient*S.T
		if (todo > 1) {
			S.H = S.T'*S.H*S.T
		}
	}
}

/*STATIC*/ `Errcode' opt__d0_h_2(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real scalar	fph,			// f(x+h)
	real scalar	fmh,			// f(x-h)
	real rowvector	g0,
	real matrix	H0
)
{
	real scalar	goal0, goal1, mgoal, mingoal, iter, itmax
	real scalar	newscale, scale1, scale2, bestscale
	real scalar	dv, dv1, dv2, bestdv
	real rowvector	p_alt

	goal0	= 1e-8
	goal1	= 1e-7
	mingoal	= 1e-12
	itmax	= 40

	goal0	= (abs(S.value)+goal0)*goal0
	goal1	= (abs(S.value)+goal1)*goal1
	mgoal	= (goal0+goal1)/2
	mingoal	= (abs(S.value)+mingoal)*mingoal

	p_alt	= S.params

	dv	= (abs(S.value - fph) + abs(S.value - fmh))/2
	bestdv	= dv
	bestscale	= scale
	scale1	= 0
	dv1	= 0
	// use interpolation to get the scaled delta to within our goals
	for (iter=1; iter<=itmax & (dv<goal0 | dv>goal1); iter++) {
		if (dv < mingoal) {
			newscale = 2*scale
		}
		else if (iter==1) {
			// linear interpolation
			opt__ipolate_linear(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1)
		}
		else {
			// quadratic interpolation
			pragma unset scale2
			pragma unset dv2
			opt__ipolate_quadratic(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1,
				scale2,		dv2)
		}

		if (missing(newscale)
		 | newscale <= 0
		 | (dv > goal1 & newscale > scale)
		 | (dv < goal0 & newscale < scale)) {
			if (dv < goal0) {
				newscale	= 2*scale
			}
			else	newscale	= scale/2
		}

		scale2	= scale1
		dv2	= dv1
		scale1	= scale
		dv1	= dv

		scale	= newscale

		p_alt[i]= S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,
			S.minimize,
			0,p_alt,fmh,g0,H0)
		if (!missing(fmh)) {
			p_alt[i]= S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fph,g0,H0)
		}
		if (missing(fmh) | missing(fph)) {
			if (bestdv >= mingoal) {
				scale	= bestscale
				p_alt[i]= S.params[i] - floatround(h*scale)
				(*S.calluser)(S.user,S.arglist,
					S.minimize,
					0,p_alt,fmh,g0,H0)
				p_alt[i]= S.params[i] + floatround(h*scale)
				(*S.calluser)(S.user,S.arglist,
					S.minimize,
					0,p_alt,fph,g0,H0)

				displayas("txt")
				printf(
	"numerical derivatives are approximate\nnearby values are missing\n"
				)
				return(0)
			}
			S.errorcode = `Errcode_discon_miss'
			S.errortext = `Errtext_discon_miss'
			opt__errorhandler(S)
			return(S.errorcode)
		}

		dv = (abs(S.value - fph) + abs(S.value - fmh))/2

		if (dv > 1.1*bestdv
		 | (dv >= 0.9*bestdv & scale < bestscale)) {
			bestdv		= dv
			bestscale	= scale
		}
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		if (bestdv >= mingoal) {
			// go with the best value
			scale	= bestscale
			p_alt[i]= S.params[i] - floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fmh,g0,H0)
			p_alt[i]= S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fph,g0,H0)

			displayas("txt")
			printf(
	"numerical derivatives are approximate\n" +
	"flat or discontinuous region encountered\n"
			)
		}
		else {
			S.errorcode = `Errcode_discon_miss'
			S.errortext = `Errtext_discon_miss'
			opt__errorhandler(S)
			return(S.errorcode)
		}
	}
	return(0)
}

/*STATIC*/ `Errcode' opt__d0_h(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	fph,
	real scalar	fmh,
	real rowvector	g0,
	real matrix	H0
)
{
	`Errcode'	ec
	real scalar	scale

	h = 1e-3
	h = (abs(S.params[i])+h)*h
	// get the numerical derivative scale for the ith parameter
	scale = S.scale[i]

	ec = opt__d0_h_size(S,i,h,scale,fph,fmh,g0,H0)
	if (ec) return(ec)

	// save the new numerical derivative scale for the ith parameter
	S.scale[i] = scale

	// return the value of the perturbation for taking a numerical
	// derivative with respect to the ith parameter
	h = floatround(h*scale)
	return(0)
}

/*STATIC*/ `Errcode' opt__d0_h_missing(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real scalar	fph,			// f(x+h)
	real scalar	fmh,			// f(x-h)
	real rowvector	g0,
	real matrix	H0
)
{
	real scalar	fm, fp
	real scalar	iter, itmax
	real rowvector	p_alt

	itmax	= 50
	fm	= .
	fp	= .

	p_alt = S.params
	for (iter=1; iter<=itmax & (missing(fm) | missing(fp)); iter++) {
		scale = scale/2
		p_alt[i] = S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,
			S.minimize,
			0,p_alt,fm,g0,H0)
		if (!missing(fm)) {
			p_alt[i] = S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fp,g0,H0)
		}
	}
	if (missing(fm) | missing(fp)) {
		S.errorcode = `Errcode_discon_miss'
		S.errortext = `Errtext_discon_miss'
		opt__errorhandler(S)
		return(S.errorcode)
	}
	fmh = fm
	fph = fp
	return(opt__d0_h_2(S,i,h,scale,fph,fmh,g0,H0))
}

/*STATIC*/ `Errcode' opt__d0_h_size(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real scalar	fph,			// f(x+h)
	real scalar	fmh,			// f(x-h)
	real rowvector	g0,
	real matrix	H0
)
{
	`Errcode'	ec
	real scalar	goal0, goal1, mgoal, mingoal, iter, itmax
	real scalar	fm0
	real scalar	scale0, newscale, scale1, scale2
	real scalar	dv, dv1, dv2
	real rowvector	p_alt

	if (S.iter == 0) {
		goal0	= 1e-4
		goal1	= 1e-3
	}
	else {
		goal0	= 1e-8
		goal1	= 1e-7
	}
	mingoal	= 1e-10
	itmax	= 20

	goal0	= (abs(S.value)+goal0)*goal0
	goal1	= (abs(S.value)+goal1)*goal1
	mgoal	= (goal0+goal1)/2
	mingoal	= (abs(S.value)+mingoal)*mingoal

	p_alt	= S.params

	p_alt[i]= S.params[i] - floatround(h*scale)
	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,p_alt,fmh,g0,H0)
	if (missing(fmh)) {
		return(opt__d0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
	}

	// Save initial values, we may have to restore them later
	scale0	= scale
	fm0	= fmh

	dv	= abs(S.value-fmh)
	scale1	= 0
	dv1	= 0
	// use interpolation to get the scaled delta to within our goals
	for (iter=1; iter<=itmax & (dv<goal0 | dv>goal1); iter++) {
		if (dv < mingoal) {
			newscale = 2*scale
		}
		else if (iter==1) {
			// linear interpolation
			opt__ipolate_linear(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1)
		}
		else {
			// quadratic interpolation
			pragma unset scale2
			pragma unset dv2
			opt__ipolate_quadratic(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1,
				scale2,		dv2)
		}

		if (missing(newscale)
		 | newscale <= 0
		 | (dv > goal1 & newscale > scale)
		 | (dv < goal0 & newscale < scale)) {
			if (dv < goal0) {
				newscale	= 2*scale
			}
			else	newscale	= scale/2
		}

		scale2	= scale1
		dv2	= dv1
		scale1	= scale
		dv1	= dv

		scale	= newscale

		p_alt[i]= S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,S.minimize,
			0,p_alt,fmh,g0,H0)
		if (missing(fmh)) {
			return(opt__d0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
		}
		dv	= abs(S.value-fmh)
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		scale	= scale0	// go back to the initial values
		fmh	= fm0		// guaranteed to be nonmissing
	}

	p_alt[i]	= S.params[i] + floatround(h*scale)
	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,p_alt,fph,g0,H0)
	if (missing(fph)) {
		return(opt__d0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		// redo the stepsize adjustment looking at both sides
		// starting values are guaranteed not to be missing
		return(opt__d0_h_2(S,i,h,scale,fph,fmh,g0,H0))
	}
	return(0)

}

/*STATIC*/ `Errcode' opt__dfp(`OptStruct' S)
{
	if (S.iter == 0) {
		S.oldparams	= S.params
		S.oldgrad	= S.gradient
		S.H	= I(S.dim)
		return(0)
	}

	real scalar	eps
	real scalar	dbdgp, dgHdgp, gPg, bPb
	real matrix	db, dg
	real matrix	V

	V	= S.H
	eps	= 1e-8
	db	= S.params - S.oldparams
	dg	= S.gradient - S.oldgrad
	dbdgp	= db * dg'
	dgHdgp	= dg * V * dg'
	bPb	= db * db'
	gPg	= dg * dg'
	if (abs(dbdgp*dgHdgp) > eps*gPg*bPb) {
		V = V - db'*db/dbdgp - (V*dg')*(dg*V')/dgHdgp
		V = (V + V')/2
		if (missing(V)) {
			S.errorcode = `Errcode_unstable'
			S.errortext = `Errtext_unstable'
			opt__errorhandler(S)
			return(S.errorcode)
		}
		S.H	= V
	}
	else {
		if (S.reset < 10) {
			if (S.trace > `OPT_trace_none') {
				displayas("txt")
	printf("DFP stepping has contracted, resetting DFP Hessian\n")
			}
			S.H	= I(S.dim)
			S.reset	= S.reset + 1
		}
		else {
			S.errorcode = `Errcode_flat'
			S.errortext = `Errtext_flat'
			opt__errorhandler(S)
			return(S.errorcode)
		}
	}

	S.oldparams	= S.params
	S.oldgrad	= S.gradient
	return(0)
}

/*STATIC*/ void opt__errorhandler(`OptStruct' S)
{
	if (!S.errorcode) return
	if (S.verbose) {
		errprintf("{p}\n")
		errprintf("%s\n", S.errortext)
		errprintf("{p_end}\n")
	}
	if (S.ucall) return
	exit(optimize_result_returncode(S))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_d0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i
	real scalar	fp, fm, hi
	real rowvector	h
	real rowvector	grad, g0
	real matrix	H0			// ignored
	pragma unset g0
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.value,g0,H0)

	if (todo == 0 | missing(S.value)) return(0)

	dim	= S.dim
	h	= J(1,dim,.)
	grad	= J(1,dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec	= opt__d0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]	= hi
		grad[i]	= (fp - fm)/(2*h[i])
	}

	S.gradient	= grad
	return(opt__bfgs(S))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_d1(`OptStruct' S, real scalar todo)
{
	real matrix	H0			// ignored
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.value,S.gradient,H0)
	if (todo != 2 | missing(S.value)) return(0)

	return(opt__bfgs(S))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_d2(`OptStruct' S, real scalar todo)
{
	return(opt__eval_bfgs_d1(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_v0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i
	real scalar	hi
	real colvector	fp, fm
	real rowvector	h
	real matrix	grad_v
	real rowvector	g0			// ignored
	real matrix	g0_v			// ignored
	real matrix	H0			// ignored
	pragma unset g0_v

	dim	= S.dim
	g0	= J(1,dim,.)
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.weights,
		S.value,g0,H0,S.value_v,g0_v)
	if (todo == 0 | missing(S.value)) return(0)

	h	= J(1,dim,.)
	grad_v	= J(rows(S.value_v),dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec = opt__v0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]		= hi
		grad_v[,i]	= (fp - fm)/(2*h[i])
	}

	S.gradient_v= grad_v
	S.gradient	= quadcross(S.weights, grad_v)
	return(opt__bfgs(S))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_v1(`OptStruct' S, real scalar todo)
{
	real matrix	H0			// ignored
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.weights,
		S.value,S.gradient,H0,
		S.value_v,S.gradient_v)
	if (todo != 2 | missing(S.value)) return(0)

	return(opt__bfgs(S))
}

/*STATIC*/ `Errcode' opt__eval_bfgs_v2(`OptStruct' S, real scalar todo)
{
	return(opt__eval_bfgs_v1(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_bhhh_v0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i
	real colvector	fp, fm
	real scalar	hi
	real rowvector	h
	real matrix	grad_v
	real rowvector	g0			// ignored
	real matrix	g0_v			// ignored
	real matrix	H0			// ignored
	pragma unset g0_v

	dim	= S.dim
	g0	= J(1,dim,.)
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.weights,
		S.value,g0,H0,S.value_v,g0_v)
	if (todo == 0 | missing(S.value)) return(0)

	h	= J(1,dim,.)
	grad_v	= J(rows(S.value_v),dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec		= opt__v0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]		= hi
		grad_v[,i]	= (fp - fm)/(2*h[i])
	}

	S.gradient_v= grad_v
	S.gradient	= quadcross(S.weights, grad_v)

	opt__bhhh(S)
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_bhhh_v1(`OptStruct' S, real scalar todo)
{
	real scalar	dim
	real matrix	H0			// ignored
	pragma unset H0

	dim	= S.dim
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.weights,
		S.value,S.gradient,H0,
		S.value_v,S.gradient_v)
	if (todo != 2 | missing(S.value)) return(0)

	opt__bhhh(S)
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_bhhh_v2(`OptStruct' S, real scalar todo)
{
	return(opt__eval_bhhh_v1(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_dfp_d0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i
	real scalar	fp, fm, hi
	real rowvector	h
	real rowvector	grad, g0
	real matrix	H0			// ignored
	pragma unset g0
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.value,g0,H0)

	if (todo == 0 | missing(S.value)) return(0)

	dim	= S.dim
	h	= J(1,dim,.)
	grad	= J(1,dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec	= opt__d0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]	= hi
		grad[i]	= (fp - fm)/(2*h[i])
	}

	S.gradient	= grad
	return(opt__dfp(S))
}

/*STATIC*/ `Errcode' opt__eval_dfp_d1(`OptStruct' S, real scalar todo)
{
	real matrix	H0			// ignored
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.value,S.gradient,H0)
	if (todo != 2 | missing(S.value)) return(0)

	return(opt__dfp(S))
}

/*STATIC*/ `Errcode' opt__eval_dfp_d2(`OptStruct' S, real scalar todo)
{
	return(opt__eval_dfp_d1(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_dfp_v0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i
	real scalar	hi
	real colvector	fp, fm
	real rowvector	h
	real matrix	grad_v
	real rowvector	g0			// ignored
	real matrix	g0_v			// ignored
	real matrix	H0			// ignored
	pragma unset g0_v

	dim	= S.dim
	g0	= J(1,dim,.)
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.weights,
		S.value,g0,H0,S.value_v,g0_v)
	if (todo == 0 | missing(S.value)) return(0)

	h	= J(1,dim,.)
	grad_v	= J(rows(S.value_v),dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec		= opt__v0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]		= hi
		grad_v[,i]	= (fp - fm)/(2*h[i])
	}

	S.gradient_v= grad_v
	S.gradient	= quadcross(S.weights, grad_v)
	return(opt__dfp(S))
}

/*STATIC*/ `Errcode' opt__eval_dfp_v1(`OptStruct' S, real scalar todo)
{
	real matrix	H0			// ignored
	pragma unset H0

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.weights,
		S.value,S.gradient,H0,
		S.value_v,S.gradient_v)
	if (todo != 2 | missing(S.value)) return(0)

	return(opt__dfp(S))
}

/*STATIC*/ `Errcode' opt__eval_dfp_v2(`OptStruct' S, real scalar todo)
{
	return(opt__eval_dfp_v1(S, todo))
}

/*STATIC*/ `Errcode' opt__eval(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec

	if (S.hasCns) {
		opt__cns_off(S, todo)
	}

	ec = (*S.eval)(S,todo)
	if (ec) return(ec)

	if (S.hasCns) {
		opt__cns_on(S, todo)
	}
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nm_d0(`OptStruct' S, real scalar todo)
{
	pragma unset todo
	return((*S.calluser)(
		S.user,
		S.arglist,
		S.minimize,
		0,
		S.params,
		S.value,
		S.gradient,
		S.H
	))
}

/*STATIC*/ void opt__eval_nm(
	`OptStruct' S,
	real rowvector	params,
	real scalar	value)
{
	if (S.hasCns) {
		params = params*S.T'+S.a
	}
	if (S.evaltype < `OPT_evaltype_v0') {
		(*S.calluser)(
			S.user,
			S.arglist,
			S.minimize,
			0,
			params,
			value,
			S.gradient,
			S.H
		)
	}
	else {
		real colvector	v_v
		real matrix	g_v
		pragma unset v_v
		pragma unset g_v
		(*S.calluser)(
			S.user,
			S.arglist,
			S.minimize,
			0,
			params,
			S.weights,
			value,
			S.gradient,
			S.H,
			v_v,
			g_v
		)
	}
	if (S.hasCns) {
		params = (params-S.a)*S.T
	}
}

/*STATIC*/ `Errcode' opt__eval_nm_v0(`OptStruct' S, real scalar todo)
{
	pragma unset todo
	return((*S.calluser)(
		S.user,
		S.arglist,
		S.minimize,
		0,
		S.params,
		S.weights,
		S.value,
		S.gradient,
		S.H,
		S.value_v,
		S.gradient_v
	))
}

/*STATIC*/ `Errcode' opt__eval_nr_d0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i, j
	real scalar	fp, fpp, fm, fmm, hi
	real rowvector	p_alt, h, fph, fmh
	real scalar	v0
	real rowvector	grad
	real matrix	Hess
	real rowvector	g0			// ignored
	real matrix	H0			// ignored

	dim	= S.dim
	g0	= J(1,dim,.)
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.value,g0,H0)

	if (todo == 0 | missing(S.value)) return(0)

	p_alt	= J(1,dim,.)
	h	= J(1,dim,.)
	fph	= J(1,dim,.)
	fmh	= J(1,dim,.)
	v0	= S.value
	grad	= J(1,dim,.)
	Hess	= J(dim,dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fpp
	pragma unset fm
	pragma unset fmm
	for (i=1; i<=dim; i++) {
		ec	= opt__d0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]	= hi
		fph[i]	= fp
		fmh[i]	= fm
		grad[i]	= (fp - fm)/(2*h[i])

		// diagonal element of the Hessian matrix
		Hess[i,i] = (fp - 2*v0 + fm)/(h[i]^2)

		// off diagonal elements of the Hessian matrix
		for (j=1; j<i; j++) {
			p_alt    = S.params
			p_alt[i] = S.params[i] + h[i]
			p_alt[j] = S.params[j] + h[j]
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fpp,g0,H0)

			p_alt[i] = S.params[i] - h[i]
			p_alt[j] = S.params[j] - h[j]
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,fmm,g0,H0)

			Hess[i,j] = (fpp + fmm + 2*v0 - fp
				- fm - fph[j] - fmh[j]) / (2*h[i]*h[j])
			Hess[j,i] = Hess[i,j]
		}
	}
	S.value	= v0
	S.gradient	= grad
	S.H		= Hess
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_d1debug(`OptStruct' S, real scalar todo)
{
	return(opt__eval_nr_d2debug(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_nr_d1(`OptStruct' S, real scalar todo)
{
	real scalar	dim, eps, h, i
	real rowvector	p_alt, gp, gm
	real scalar	v0			// ignored
	real matrix	H0			// ignored
	pragma unset v0
	pragma unset H0

	dim	= S.dim
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.value,S.gradient,H0)
	if (todo != 2 | missing(S.value)) return(0)

	p_alt	= J(1,dim,.)
	gp	= J(1,dim,.)
	gm	= J(1,dim,.)
	eps	= 1e-4
	for (i=1; i<=dim; i++) {
		h = (abs(S.params[i]) + eps)*eps
		p_alt    = S.params
		p_alt[i] = S.params[i] + h
		(*S.calluser)(S.user,S.arglist,S.minimize,
			1,p_alt,v0,gp,H0)

		p_alt[i] = S.params[i] - h
		(*S.calluser)(S.user,S.arglist,S.minimize,
			1,p_alt,v0,gm,H0)
		S.H[i,] = (gp - gm)/(2*h)
	}
	S.H = (S.H + S.H')/2
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_d2debug(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec

	// call d0 method, and exit if no derivatives were requested
	ec = opt__eval_nr_d0(S, todo)
	if (ec) return(ec)
	if (todo == 0) return(0)

	real scalar	dim
	real scalar	v
	real rowvector	p, g
	real matrix	H
	pragma unset v

	dim	= S.dim
	p	= S.params
	g	= J(1,dim,.)
	H	= J(dim,dim,.)
	// call user to compute some derivatives to compare with d0
	if (S.evaltype == `OPT_evaltype_d2debug') {
		(*S.calluser)(S.user,S.arglist,S.minimize,todo,p,v,g,H)
	}
	else {
		(*S.calluser)(S.user,S.arglist,S.minimize,todo>0,p,v,g,H)
	}

	opt__eval_nr_debug_report(S,todo,p,v,g,H)
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_d2(`OptStruct' S, real scalar todo)
{
	(*S.calluser)(
		S.user,
		S.arglist,
		S.minimize,
		todo,
		S.params,
		S.value,
		S.gradient,
		S.H
	)
	return(0)
}

/*STATIC*/ void opt__eval_nr_debug_report(
	`OptStruct' S,
	real scalar	todo,
	real rowvector	p,
	real scalar	v,
	real rowvector	g,
	real matrix	H

)
{
	string scalar	evaltype
	real scalar	dim, trace

	evaltype= optimize_init_evaluatortype(S)
	dim	= S.dim
	trace	= S.trace

	if (trace) {
		displayas("txt")
		if (S.iter) {
			printf("\n")
		}
		printf("%s:  Begin derivative-comparison report {hline}\n",
			evaltype)
	}

	// check the parameter vector
	if (mreldif(p,S.params) != 0) {
		printf("%s:  Warning:  the coefficient vector was damaged\n\n",
			evaltype)
		printf("\n%s:  Coefficient vector was\n", evaltype)
		S.params
		printf("\n%s:  After calling your evaluator, " +
		       "it changed to\n", evaltype)
		p
		printf("\n")
	}

	// check the value
	if (v != S.value) {
		printf("%s:  Warning:  " +
			"the evaluator returned different values\n", evaltype)
		printf("%s:  First it returned {res:%10.0g}\n",
			evaltype,
			(S.minimize ? -S.value : S.value))
		printf("%s:  Then it returned {res:%10.0g}\n",
			evaltype,
			(S.minimize ? -v : v))
	}
	if (missing(S.value) & missing(v)) {
		printf("%s:  evaluator returned missing value\n", evaltype)
		printf("%s:  End derivative-comparison report {hline}\n\n",
			evaltype)
		return
	}

	// check the gradient vector
	if (missing(g)) {
		printf("%s:  Warning:  " +
			"evaluator did not compute gradient vector\n",
			evaltype)
	}
	else if (trace) {
		if (cols(S.gradient) == cols(g)
		 &  rows(S.gradient) == rows(g)) {
			printf("%s:  mreldif(gradient vectors) = {res:%9.0g}\n",
				evaltype, mreldif(S.gradient,g))
			if (trace >= `OPT_trace_gradient') {
				printf("\n%s:  analytically calculated " +
					"gradient:\n",
					evaltype)
				if (S.minimize) {
					-g
				}
				else	g
				printf("\n%s:  numerically calculated " +
					"gradient (used for stepping):\n",
					evaltype)
				if (S.minimize) {
					-S.gradient
				}
				else	S.gradient
				printf("\n%s:  relative differences:\n",
					evaltype)
				reldif(S.gradient,g)
				printf("\n")
			}
		}
		else {
			printf("%s:  Warning:  evaluator computed a " +
				"{res:%1.0f} x {res:%1.0f} gradient matrix\n",
				evaltype, rows(g), cols(g))
			printf("%s:  gradient should be " +
				"{res:1} x {res:%1.0f} rowvector\n",
				evaltype, dim)
		}
	}
	if (todo != 2 |
	    S.evaltype == `OPT_evaltype_d1debug' |
	    S.evaltype == `OPT_evaltype_v1debug') {
		if (trace) {
			printf(
			"%s:  End derivative-comparison report {hline}\n\n",
				evaltype)
		}
		return
	}

	// check the Hessian matrix
	if (missing(H)) {
		printf("%s:  Warning:  " +
			"evaluator did not compute Hessian matrix\n",
			evaltype)
	}
	else if (trace) {
		if (cols(S.H) == cols(H)
		 &  rows(S.H) == rows(H)) {
			printf("%s:  mreldif(Hessian matrices) = {res:%9.0g}\n",
				evaltype, mreldif(S.H,H))
			if (trace >= `OPT_trace_hessian') {
				printf("\n%s:  analytically calculated " +
					"Hessian:\n",
					evaltype)
				if (S.minimize) {
					-H
				}
				else	H
				printf("\n%s:  numerically calculated " +
					"Hessian (used for stepping):\n",
					evaltype)
				if (S.minimize) {
					-S.H
				}
				else	S.H
				printf("\n%s:  relative differences:\n",
					evaltype)
				reldif(S.H,H)
				printf("\n")
			}
		}
		else {
			printf("%s:  Warning:  evaluator computed a " +
				"{res:%1.0f} x {res:%1.0f} Hessian matrix\n",
				evaltype, rows(H), cols(H))
			printf("%s:  Hessian should be " +
				"{res:%1.0f} x {res:%1.0f} " +
				"matrix\n", evaltype, dim, dim)
		}
	}
	if (trace) {
		printf("%s:  End derivative-comparison report {hline}\n",
			evaltype)
	}
}

/*STATIC*/ `Errcode' opt__eval_nr_v0(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec
	real scalar	dim, i, j
	real scalar	hi
	real colvector	fp, fm
	real rowvector	p_alt, h, fph, fmh
	real scalar	v0
	real matrix	grad_v, Hess
	real colvector	v0_v			// ignored
	real rowvector	g0			// ignored
	real matrix	g0_v			// ignored
	real matrix	H0			// ignored
	pragma unset v0_v
	pragma unset g0_v

	dim	= S.dim
	g0	= J(1,dim,.)
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,S.params,S.weights,
		S.value,g0,H0,S.value_v,g0_v)
	if (todo == 0 | missing(S.value)) return(0)

	p_alt	= J(1,dim,.)
	h	= J(1,dim,.)
	fph	= J(1,dim,.)
	fmh	= J(1,dim,.)
	v0	= S.value
	grad_v	= J(rows(S.value_v),dim,.)
	Hess	= J(dim,dim,.)

	pragma unset hi
	pragma unset fp
	pragma unset fm
	for (i=1; i<=dim; i++) {
		ec	= opt__v0_h(S,i,hi,fp,fm,g0,H0)
		if (ec) return(ec)
		h[i]	= hi
		fph[i]	= quadcross(S.weights, fp)
		fmh[i]	= quadcross(S.weights, fm)
		grad_v[,i] = (fp - fm)/(2*h[i])

		// diagonal element of the Hessian matrix
		Hess[i,i] = (fph[i] - 2*v0 + fmh[i])/(h[i]^2)

		// NOTE: 'fp' and 'fm' are scalars in the following loop
		fp = .
		fm = .
		// off diagonal elements of the Hessian matrix
		for (j=1; j<i; j++) {
			p_alt    = S.params
			p_alt[i] = S.params[i] + h[i]
			p_alt[j] = S.params[j] + h[j]
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,
				fp,g0,H0,v0_v,g0_v)

			p_alt[i] = S.params[i] - h[i]
			p_alt[j] = S.params[j] - h[j]
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,
				fm,g0,H0,v0_v,g0_v)

			Hess[i,j] = (fp + fm + 2*v0 - fph[i]
				- fmh[i] - fph[j] - fmh[j]) / (2*h[i]*h[j])
			Hess[j,i] = Hess[i,j]
		}
	}
	S.value	= v0
	S.gradient_v= grad_v
	S.gradient	= quadcross(S.weights, grad_v)
	S.H	= Hess
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_v1debug(`OptStruct' S, real scalar todo)
{
	return(opt__eval_nr_v2debug(S, todo))
}

/*STATIC*/ `Errcode' opt__eval_nr_v1(`OptStruct' S, real scalar todo)
{
	real scalar	dim, eps, h, i
	real rowvector	p_alt, gp, gm
	real matrix	gp_v, gm_v
	real scalar	v0			// ignored
	real colvector	v0_v			// ignored
	real matrix	H0			// ignored
	pragma unset gm_v
	pragma unset gp_v
	pragma unset v0
	pragma unset v0_v
	pragma unset H0

	dim	= S.dim
	H0	= J(dim,dim,.)

	(*S.calluser)(S.user,S.arglist,S.minimize,
		todo>0,S.params,S.weights,
		S.value,S.gradient,H0,
		S.value_v,S.gradient_v)
	if (todo != 2 | missing(S.value)) return(0)

	p_alt	= J(1,dim,.)
	gp	= J(1,dim,.)
	gm	= J(1,dim,.)
	eps	= 1e-4
	for (i=1; i<=dim; i++) {
		h = (abs(S.params[i]) + eps)*eps
		p_alt    = S.params
		p_alt[i] = S.params[i] + h
		(*S.calluser)(S.user,S.arglist,S.minimize,
			1,p_alt,S.weights,
			v0,gp,H0,v0_v,gp_v)

		p_alt[i] = S.params[i] - h
		(*S.calluser)(S.user,S.arglist,S.minimize,
			1,p_alt,S.weights,
			v0,gm,H0,v0_v,gm_v)
		S.H[i,] = (gp - gm)/(2*h)
	}
	S.H = (S.H + S.H')/2
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_v2debug(`OptStruct' S, real scalar todo)
{
	`Errcode'	ec

	// call d0 method, and exit if no derivatives were requested
	ec = opt__eval_nr_v0(S, todo)
	if (ec) return(ec)
	if (todo == 0) return(0)

	real scalar	dim
	real scalar	v
	real colvector	v_v
	real rowvector	p, g
	real matrix	g_v
	real matrix	H
	pragma unset v
	pragma unset v_v
	pragma unset g_v

	dim	= S.dim
	p	= S.params
	g	= J(1,dim,.)
	H	= J(dim,dim,.)
	// call user to compute some derivatives to compare with d0
	if (S.evaltype == `OPT_evaltype_v2debug') {
		(*S.calluser)(S.user,S.arglist,S.minimize,
			todo,p,S.weights,
			v,g,H,v_v,g_v)
	}
	else {
		(*S.calluser)(S.user,S.arglist,S.minimize,
			todo>0,p,S.weights,
			v,g,H,v_v,g_v)
	}

	opt__eval_nr_debug_report(S,todo,p,v,g,H)
	return(0)
}

/*STATIC*/ `Errcode' opt__eval_nr_v2(`OptStruct' S, real scalar todo)
{
	(*S.calluser)(
		S.user,
		S.arglist,
		S.minimize,
		todo,
		S.params,
		S.weights,
		S.value,
		S.gradient,
		S.H,
		S.value_v,
		S.gradient_v
	)
	return(0)
}

/*STATIC*/ function opt__evaltype_num(`OptStruct' S, |real scalar type)
{
	if (args() == 2) {
		opt__set_int(S.evaltype, type, 1, `OPT_evaltype_max')
		S.evaltype = type
	}
	else	return(S.evaltype)
}

/*STATIC*/ function opt__evaltype_str(`OptStruct' S, |string scalar type)
{
	if (args() == 2) {
		if (type == `OPT_evaltype1') {
			S.evaltype = 1
		}
		else if (type == `OPT_evaltype2') {
			S.evaltype = 2
		}
		else if (type == `OPT_evaltype3') {
			S.evaltype = 3
		}
		else if (type == `OPT_evaltype4') {
			S.evaltype = 4
		}
		else if (type == `OPT_evaltype5') {
			S.evaltype = 5
		}
		else if (type == `OPT_evaltype6') {
			S.evaltype = 6
		}
		else if (type == `OPT_evaltype7') {
			S.evaltype = 7
		}
		else if (type == `OPT_evaltype8') {
			S.evaltype = 8
		}
		else if (type == `OPT_evaltype9') {
			S.evaltype = 9
		}
		else if (type == `OPT_evaltype10') {
			S.evaltype = 10
		}
		else {
			_error("invalid evaluator type")
			/*NOTREACHED*/
		}
	}
	else {
		if (S.evaltype == 1) {
			return(`OPT_evaltype1')
		}
		if (S.evaltype == 2) {
			return(`OPT_evaltype2')
		}
		if (S.evaltype == 3) {
			return(`OPT_evaltype3')
		}
		if (S.evaltype == 4) {
			return(`OPT_evaltype4')
		}
		if (S.evaltype == 5) {
			return(`OPT_evaltype5')
		}
		if (S.evaltype == 6) {
			return(`OPT_evaltype6')
		}
		if (S.evaltype == 7) {
			return(`OPT_evaltype7')
		}
		if (S.evaltype == 8) {
			return(`OPT_evaltype8')
		}
		if (S.evaltype == 9) {
			return(`OPT_evaltype9')
		}
		if (S.evaltype == 10) {
			return(`OPT_evaltype10')
		}
	}
}

/*STATIC*/ void opt__init(`OptStruct' S)
{
	// items for the optimizer status
	S.valid		= `OPT_valid_default'
	S.eval		= `OPT_eval_default'
	S.dim		= `OPT_dim_default'
	S.concave	= `OPT_concave_default'
	S.backup	= `OPT_backup_default'
	S.scale		= `OPT_scale_default'
	S.constraints	= `OPT_constraints_default'
	S.T		= `OPT_T_default'
	S.a		= `OPT_a_default'
	S.eigen		= `OPT_eigen_default'

	S.reset		= `OPT_reset_default'

	// items with output routines for the user
	S.value		= `OPT_value_default'
	S.value_v	= `OPT_value_v_default'
	S.params	= `OPT_params_default'
	S.gradient	= `OPT_gradient_default'
	S.gradient_v	= `OPT_gradient_v_default'
	S.H		= `OPT_H_default'
	S.iter		= `OPT_iter_default'
	S.converged	= `OPT_converged_default'
	S.convtol	= `OPT_convtol_default'
	S.log		= `OPT_log_default'

	// items for the simplex used by NM
	S.simplex	= `OPT_simplex_default'
	S.simvals	= `OPT_simvals_default'
	S.simorder	= `OPT_simorder_default'
	S.simnodes	= `OPT_simnodes_default'

	S.oerrorcode	= `OPT_oerrorcode_default'

	S.verbose	= `OPT_verbose_default'
	S.ucall		= `OPT_ucall_default'
}

/*STATIC*/ void opt__ipolate_linear(
	real scalar	y,
	real scalar	x,
	real scalar	y0,
	real scalar	x0,
	real scalar	y1,
	real scalar	x1
)
{
	y = y0 + (y1 - y0)*(x - x0)/(x1 - x0)
}

/*STATIC*/ void opt__ipolate_quadratic(
	real scalar	y,
	real scalar	x,
	real scalar	y0,
	real scalar	x0,
	real scalar	y1,
	real scalar	x1,
	real scalar	y2,
	real scalar	x2
)
{
	y =	y0*(x - x1)*(x - x2)/((x0 - x1)*(x0 - x2)) +
		y1*(x - x0)*(x - x2)/((x1 - x0)*(x1 - x2)) +
		y2*(x - x0)*(x - x1)/((x2 - x0)*(x2 - x1))
}

/*STATIC*/ void opt__log_value(`OptStruct' S)
{
	real	scalar	i

	i	= mod(S.iter, 20) + 1
	if (missing(S.value)) {
		S.log[i]	= 0
	}
	else	S.log[i]	= S.value
	S.iter = S.iter + 1
}

/*STATIC*/ `Errcode' opt__loop_bfgs(`OptStruct' S)
{
	return(opt__loop_dfp(S))
}

/*STATIC*/ `Errcode' opt__loop_bhhh(`OptStruct' S)
{
	`Errcode'	ec

	// check for evaltypes not allowed with BHHH
	if (S.evaltype <= `OPT_evaltype_d2debug') {
		_error(sprintf(
		"%s evaluators are not allowed with the bhhh technique",
			optimize_init_evaluatortype(S)))
		/*NOTREACHED*/
	}

	ec = opt__loop_nr(S)
	if (ec) return(ec)

	S.hessian = `OPT_hessian_default'
	return(0)
}

/*STATIC*/ `Errcode' opt__loop_dfp(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	iter, maxiter

	S.reset		= `OPT_reset_default'
	S.iter		= `OPT_iter_default'
	S.convtol	= `OPT_convtol_default'
	S.converged	= `OPT_converged_default'

	maxiter	= S.maxiter

	if (S.hasCns) {
		S.params = S.params*S.T
		opt__cns_off(S, 0)
	}

	if (S.trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", 0)

		if (S.trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	ec = (*S.eval)(S,2)
	if (ec) return(ec)
	if (missing(S.value)) {
		S.errorcode = `Errcode_notfeasible'
		S.errortext = `Errtext_notfeasible'
		opt__errorhandler(S)
		return(S.errorcode)
	}
	// log value and step iteration count
	opt__log_value(S)

	if (S.hasCns) {
		opt__cns_on(S, 2)
	}

	// save the value from 'Iteration 0:'
	S.v0 = S.value

	for (iter=1; iter <= maxiter & !S.converged; ++iter) {
		ec = (*S.stepper)(S)
		if (ec) return(ec)
	}

	opt__print_log(S)

	if (S.hasCns) {
		opt__cns_off(S, 2)
	}

	if (S.trace > `OPT_trace_tol') {
		printf("{hline}\n")
	}

	return(0)
}

/*STATIC*/ `Errcode' opt__loop_nm(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	iter, maxiter
	real scalar	i
	real scalar	simnodes
	real scalar	newv

	S.iter		= `OPT_iter_default'
	S.converged	= `OPT_converged_default'

	maxiter		= S.maxiter
	S.concave	= 1

	if (S.hasCns) {
		S.params = S.params*S.T
		opt__cns_off(S, 0)
		opt__cns_on(S, 0)
	}

	if (S.trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", 0)

		if (S.trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	S.simplex	= opt__simplex(S)
	S.simnodes	= rows(S.simplex)
	simnodes	= S.simnodes
	S.simvals	= J(simnodes,1,.)

	pragma unset newv
	for (i=1; i<=simnodes; i++) {
		opt__eval_nm(S, S.simplex[i,], newv)
		S.simvals[i] = newv
	}

	if (missing(S.simvals)) {
		S.errorcode = `Errcode_value_missing'
		S.errortext = `Errtext_value_missing'
		opt__errorhandler(S)
		return(S.errorcode)
		/*NOTREACHED*/
	}

	// starting values are stored in the last row of the simplex
	S.value	= S.simvals[simnodes]

	// log value and step iteration count
	opt__log_value(S)

	S.simorder	= order(S.simvals, 1)

	// save the value from 'Iteration 0:'
	S.v0 = S.value

	for (iter=1; iter <= maxiter & !S.converged; ++iter) {
		ec = (*S.stepper)(S)
		if (ec) return(ec)
	}

	if (S.hasCns) {
		opt__cns_off(S, 2)
	}

	opt__print_log(S)

	if (S.trace > `OPT_trace_tol') {
		printf("{hline}\n")
	}

	return(0)
}

/*STATIC*/ `Errcode' opt__loop_nr(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	iter, maxiter

	S.iter		= `OPT_iter_default'
	S.convtol	= `OPT_convtol_default'
	S.converged	= `OPT_converged_default'

	maxiter	= S.maxiter

	if (S.hasCns) {
		S.params = S.params*S.T
		opt__cns_off(S, 0)
	}

	if (S.trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", 0)

		if (S.trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	ec = (*S.eval)(S,2)
	if (ec) return(ec)
	if (missing(S.value)) {
		S.errorcode = `Errcode_notfeasible'
		S.errortext = `Errtext_notfeasible'
		opt__errorhandler(S)
		return(S.errorcode)
	}
	// log value and step iteration count
	opt__log_value(S)

	if (S.hasCns) {
		opt__cns_on(S, 2)
	}

	// save the value from 'Iteration 0:'
	S.v0 = S.value

	for (iter=1; iter <= maxiter & !S.converged; ++iter) {
		ec = (*S.stepper)(S)
		if (ec) return(ec)
	}

	opt__print_log(S)

	if (S.hasCns) {
		opt__cns_off(S, 2)
	}

	if (S.trace > `OPT_trace_tol') {
		printf("{hline}\n")
	}

	S.hessian	= S.H
	return(0)
}

/*STATIC*/ void opt__print_hessian(
	real	matrix	H,
	real	scalar	minimize
)
{
	displayas("txt")
	printf("Hessian matrix:\n")
	(minimize ? -H : H)
	printf("\n")
}

/*STATIC*/ void opt__print_log(`OptStruct' S)
{
	real	scalar	iter
	string	scalar	msg
		scalar	trace

	trace = S.trace
	if (trace == `OPT_trace_none')	return

	if (!S.concave) {
		msg = "  (not concave)"
	}
	else if (S.backup >= 6) {
		msg = "  (backed up)"
	}
	else	msg = ""

	iter	= S.iter - 1
	opt__print_value(
		iter,
		S.value_id,
		S.value,
		S.minimize,
		msg,
		(trace > `OPT_trace_tol')
	)

	if (trace >= `OPT_trace_tol') {
		opt__print_tol(
			iter,
			S.value_id,
			S.convtol_id,
			S.convtol,
			(trace > `OPT_trace_tol')
		)
	}

	if (trace < `OPT_trace_step' | S.technique == `OPT_technique_nm') {
		return
	}

	opt__print_vector(
		S.gradient,
		"Gradient",
		S.trace >= `OPT_trace_gradient',
		S.minimize
	)

	if (trace < `OPT_trace_hessian') {
		return
	}

	opt__print_hessian(S.H, S.minimize)
}

/*STATIC*/ void opt__print_tol(
	real	scalar	iter,
	string	scalar	value_id,
	string	scalar	tol_id,
	real	scalar	tol,
	real	scalar	flushright
)
{
	if (missing(tol)) return
	displayas("txt")
	if (flushright == 0) {
		real scalar col, spaces
		col = strlen(sprintf("Iteration %f:  ", iter)) + 1
		spaces = strlen(value_id) - strlen(tol_id) + 1
		if (spaces < 1) {
			spaces = 1
		}
		printf("{col %f}%s{space %f}= {res:%10.0g}\n",
			col, tol_id, spaces, tol)
	}
	else {
		printf("{right:%s = {res:%10.8g}}\n", tol_id, tol)
	}
}

/*STATIC*/ void opt__print_value(
	real	scalar	iter,
	string	scalar	value_id,
	real	scalar	value,
	real	scalar	minimize,
	string	scalar	msg,
	real	scalar	flushright
)
{
	displayas("txt")
	if (flushright == 0) {
		printf("Iteration %f:  %s = {res:%10.8g}%s\n",
			iter,
			value_id,
			(minimize ? -value : value),
			msg
		)
	}
	else {
		printf("{right:%s = {res:%10.8g}}\n",
			value_id,
			(minimize ? -value : value)
		)
		if (msg != "") {
			printf("{right:%s}\n", msg)
		}
	}
}

/*STATIC*/ void opt__print_vector(
	real	rowvector	g,
	string	scalar		name,
	real	scalar		showvector,
	real	scalar		minimize
)
{
	displayas("txt")
	if (!showvector) {
		printf("%s length{col 20}= {res:%9.0g}\n", name, norm(g))
	}
	else {
		printf("%s vector (length = {res:%9.0g}):\n", name, norm(g))
		(minimize ? -g : g)
		printf("\n")
	}
}

/*STATIC*/ void opt__set_int(
	real scalar opt,
	real scalar value, 
	real scalar min,
	real scalar max
)
{
	if (value<min || value>max) {
		_error("setting out of range")
		/*NOTREACHED*/
	}
	if (value != trunc(value)) {
		_error("setting must be integer")
		/*NOTREACHED*/
	}
	opt = value
}

/*STATIC*/ real matrix opt__simplex(`OptStruct' S)
{
	real scalar	dim, dimp1
	real matrix	simplex

	dim	= cols(S.params)
	dimp1	= dim + 1
	simplex	= J(dimp1,dim,.)
	if (cols(S.simplex_delta) == 1) {
		simplex[|1,1\dim,dim|] = (S.params :+ S.simplex_delta*I(dim))
	}
	else {
		simplex[|1,1\dim,dim|] = (S.params :+ diag(S.simplex_delta))
	}
	simplex[|dimp1,1\dimp1,dim|] = S.params
	return(simplex)
}

/*STATIC*/ `Errcode' opt__step_bfgs_hybrid(`OptStruct' S)
{
	return(opt__step_dfp_hybrid(S))
}

/*STATIC*/ `Errcode' opt__step_bfgs_m_marquardt(`OptStruct' S)
{
	return(opt__step_dfp_m_marquardt(S))
}

/*STATIC*/ `Errcode' opt__step_bhhh_hybrid(`OptStruct' S)
{
	return(opt__step_nr_hybrid(S))
}

/*STATIC*/ `Errcode' opt__step_bhhh_m_marquardt(`OptStruct' S)
{
	return(opt__step_nr_m_marquardt(S))
}

/*STATIC*/ `Errcode' opt__step_dfp_hybrid(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	ptol, vtol, nrtol
	real scalar	v0, v00
	real scalar	dim, k
	     scalar	trace
	real rowvector	p0, p00, gx, delta
	real matrix	invH, X, Xk, L
	pragma unset invH
	pragma unset X
	pragma unset L

	ptol		= S.ptol
	vtol		= S.vtol
	nrtol		= S.nrtol
	v00 = v0	= S.value
	p00 = p0	= S.params

	trace		= S.trace

	invH = S.H
	S.concave = opt__check_concave(S.H,invH)

	// report the value from the original parameters before we compute
	// the value at the next step
	opt__print_log(S)

	if (S.concave) {
		delta = S.gradient*invH
		S.params = p0 + delta
		if (trace >= `OPT_trace_step') {
			opt__print_vector(
				delta,
				"Step",
				trace >= `OPT_trace_params',
				S.minimize
			)
			displayas("txt")
			printf("Parameters + step -> new parameters\n")
			if (trace >= `OPT_trace_params') {
				S.params
				printf("\n")
			}
		}
	}
	else {
		symeigensystem(S.H, X, L)
		if (trace >= `OPT_trace_step') {
			displayas("txt")
			printf(
	"Dividing space into concave/nonconcave regions based on" +
	" eigenvalues\nof negative Hessian\n")
			if (trace > `OPT_trace_step') {
				printf(":\n")
				L
				printf("\n")
			}
		}
		gx	= S.gradient*X
		dim	= cols(L)
		if (L[1] > 0 & dim >=2) {
			// work with eigenvectors associated with
			// reasonably large eigenvalues
			k = min((sum(L:>=S.eigen*L[1]),dim-1) )
			Xk	= X[|1,1 \ dim,k|]
			delta	= gx[|1,1 \ 1,k|] *
				invsym(diag(L[|1,1 \ 1,k|]))*Xk'
			S.params = p0 + delta
			if (trace >= `OPT_trace_step') {
				opt__print_vector(
					delta,
	sprintf("Concave-space (dim = {res:%f}) Newton-Raphson step", k),
					trace >= `OPT_trace_params',
					S.minimize
				)
				displayas("txt")
				printf("Parameters + step -> new parameters\n")
				if (trace >= `OPT_trace_params') {
					S.params
					printf("\n")
				}
			}
			ec = opt__eval(S,0)
			if (ec) return(ec)
			if (missing(S.value) | S.value < v0) {
				if (trace >= `OPT_trace_step') {
					opt__print_value(
						.,
						S.value_id,
						S.value,
						S.minimize,
						"(initial step bad)",
						1
					)
				}
				ec = opt__steputil_backward(S,v0,p0,60)
				if (ec) return(ec)
				v0	= S.value
				p0	= S.params
			}
			else {
				if (trace >= `OPT_trace_step') {
					opt__print_value(
						.,
						S.value_id,
						S.value,
						S.minimize,
						"(initial step good)",
						1
					)
				}
				ec = opt__steputil_forward(S,delta,1/8,2)
				if (ec) return(ec)
				v0	= S.value
				p0	= S.params
			}
		}
		else	k = 0
		// work with eigenvectors associated with small or
		// negative eigenvalues
		k++
		delta	= gx[|1,k \ 1,dim|]
		Xk	= X[|1,k \ dim,dim|]
		S.value = v00
		opt__steputil_size(S, delta, delta)
		S.value = v0
		delta	= delta*Xk'
		// update parameters
		S.params = p0 + delta
		if (trace >= `OPT_trace_step') {
			opt__print_vector(
				delta,
	sprintf("\nNonconcave-space (dim = {res:%f}) steepest-ascent step",
					dim-k+1),
				trace >= `OPT_trace_params',
				S.minimize
			)
			displayas("txt")
			printf("Parameters + step -> new parameters\n")
			if (trace >= `OPT_trace_params') {
				S.params
				printf("\n")
			}
		}
	}

	// evaluate function at new parameters
	ec = opt__eval(S,0)
	if (ec) return(ec)

	if (missing(S.value) | S.value < v0) {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step bad)",
				1
			)
		}
		ec = opt__steputil_backward(S,v0,p0,(S.concave? 60 : 10))
		if (ec) return(ec)
		if (!S.concave) {
			if (missing(S.value) | S.value < v0) {
				S.params = p0
			}
		}
	}
	else {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step good)",
				1
			)
		}
		if (S.concave) {
			ec = opt__steputil_forward(S,delta,1/8,2)
		}
		else {
			ec = opt__steputil_forward(S,delta,1,2)
		}
		if (ec) return(ec)
		S.backup = `OPT_backup_default'
	}

	if (trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", S.iter)

		if (trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	// evaluate function and derivatives at new parameters
	ec = opt__eval(S,2)
	if (ec) return(ec)

	if (missing(S.value)) {
		S.errorcode = `Errcode_discon'
		S.errortext = `Errtext_discon'
		opt__errorhandler(S)
		return(S.errorcode)
	}

	// check for convergence
	if ((mreldif(S.params, p00) < ptol)
	 |  ( reldif(S.value,  v00) < vtol)) {
		if (S.concave) {
			delta	= S.gradient*S.H
			if (missing(delta)) {
				S.errorcode = `Errcode_H_notdefinite'
				if (S.minimize) {
					S.errortext = `Errtext_H_notPSD'
				}
				else {
					S.errortext = `Errtext_H_notNSD'
				}
				opt__errorhandler(S)
				return(S.errorcode)
			}
			S.convtol	= abs(S.gradient*delta')
			S.converged	= S.convtol < nrtol
		}
		else	S.convtol = `OPT_convtol_default'
	}

	// log value and step iteration count
	opt__log_value(S)

	return(0)
}

/*STATIC*/ `Errcode' opt__step_dfp_m_marquardt(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	ptol, vtol, nrtol
	real scalar	v0
	real scalar	ok, adj, tr0
	     scalar	trace
	real rowvector	p0, delta
	real matrix	invH
	pragma unset invH

	ptol		= S.ptol
	vtol		= S.vtol
	nrtol		= S.nrtol
	v0		= S.value
	p0		= S.params

	trace		= S.trace

	invH = S.H
	S.concave = opt__check_concave(S.H,invH)

	// report the value from the original parameters before we compute
	// the value at the next step
	opt__print_log(S)

	if (S.concave) {
		delta = S.gradient*invH
		S.params = p0 + delta
	}
	else {
		ok = 0
		adj = 1.1
		tr0 = trace(invH)
		while (!ok) {
			S.H = S.H - adj*abs(diag(diagonal(S.H)))
			invH	= S.H
			ok	= opt__check_concave(S.H,invH)
			adj	= 2*adj
		}
		adj = tr0/trace(invH)
		if (adj > 0) {
			invH	= adj*invH
		}
		delta = S.gradient*invH
		if (missing(delta)) {
			S.errorcode = `Errcode_H_notdefinite'
			if (S.minimize) {
				S.errortext = `Errtext_H_notPSD'
			}
			else {
				S.errortext = `Errtext_H_notNSD'
			}
			opt__errorhandler(S)
			return(S.errorcode)
		}
		// not concave
		opt__steputil_size(S, S.gradient, delta)
		// update parameters
		S.params = p0 + delta
	}

	if (trace >= `OPT_trace_step') {
		opt__print_vector(
			delta,
			"Step",
			trace >= `OPT_trace_params',
			S.minimize
		)
		displayas("txt")
		printf("Parameters + step -> new parameters\n")
		if (trace >= `OPT_trace_params') {
			S.params
			printf("\n")
		}
	}

	// evaluate function at new parameters
	ec = opt__eval(S,0)
	if (ec) return(ec)

	if (missing(S.value) | S.value < v0) {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step bad)",
				1
			)
		}
		ec = opt__steputil_backward(S,v0,p0,60)
		if (ec) return(ec)
	}
	else {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step good)",
				1
			)
		}
		if (S.concave) {
			ec = opt__steputil_forward(S,delta,1/8,2)
		}
		else {
			ec = opt__steputil_forward(S,delta,1,2)
		}
		if (ec) return(ec)
		S.backup = `OPT_backup_default'
	}

	if (trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", S.iter)

		if (trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	// evaluate function and derivatives at new parameters
	ec = opt__eval(S,2)
	if (ec) return(ec)

	if (missing(S.value)) {
		S.errorcode = `Errcode_discon'
		S.errortext = `Errtext_discon'
		opt__errorhandler(S)
		return(S.errorcode)
	}

	// check for convergence
	if ((mreldif(S.params, p0) < ptol)
	 |  ( reldif(S.value,  v0) < vtol)) {
		if (S.concave) {
			delta	= S.gradient*S.H
			if (missing(delta)) {
				S.errorcode = `Errcode_H_notdefinite'
				if (S.minimize) {
					S.errortext = `Errtext_H_notPSD'
				}
				else {
					S.errortext = `Errtext_H_notNSD'
				}
				opt__errorhandler(S)
				return(S.errorcode)
			}
			S.convtol	= abs(S.gradient*delta')
			S.converged	= S.convtol < nrtol
		}
		else	S.convtol = `OPT_convtol_default'
	}

	// log value and step iteration count
	opt__log_value(S)

	return(0)
}

/*STATIC*/ `Errcode' opt__step_nm_nm(`OptStruct' S)
{
	real scalar	i
	real scalar	c1, c2, c3
	real scalar	simnodes
	real rowvector	middle
	real rowvector	newp, altp
	real scalar	newv, altv
	real scalar	mult, better
	real scalar	ptol, vtol
	     scalar	trace

	pragma unset newv
	pragma unset altv

	trace		= S.trace

	simnodes	= S.simnodes
	c1		= S.simorder[simnodes]
	c2		= S.simorder[2]
	c3		= S.simorder[1]

	middle	= (colsum(S.simplex) - S.simplex[c3,])/S.dim
	newp	= 2*middle - S.simplex[c3,]

	// report the value from the original parameters before we compute
	// the value at the next step
	opt__print_log(S)

	opt__eval_nm(S, newp, newv)

	if (missing(newv)) {
		S.errorcode = `Errcode_H_notdefinite'
		if (S.minimize) {
			S.errortext = `Errtext_H_notPSD'
		}
		else {
			S.errortext = `Errtext_H_notNSD'
		}
		opt__errorhandler(S)
		return(S.errorcode)
	}

	// improve the worst node
	if (newv > S.simvals[c3]) {
		S.simplex[c3,]	= newp
		S.simvals[c3]	= newv
	}
	if (newv > S.simvals[c1]) {
		better	= 1
		mult	= 2
		while (better) {
			altp	= mult*newp - middle
			opt__eval_nm(S, altp, altv)
			if (!missing(altv) & altv > newv) {
				S.simplex[c3,]	= altp
				S.simvals[c3]	= altv
				newv			= altv
				mult			= 2*mult
			}
			else	better			= 0
		}
	}
	else if (newv <= S.simvals[c2]) {
		// contract the S.simplexlex toward the best node
		for (i=1; i<=simnodes; i++) {
			if (i != c1) {
				altp = (S.simplex[i,]+S.simplex[c1,])/2
				opt__eval_nm(S, altp, altv)
				S.simplex[i,]	= altp
				S.simvals[i]	= altv
			}
		}
	}
	if (missing(S.simvals)) {
		S.errorcode = `Errcode_value_missing'
		S.errortext = `Errtext_value_missing'
		opt__errorhandler(S)
		return(S.errorcode)
	}

	S.simorder	= order(S.simvals, 1)
	c1	= S.simorder[simnodes]
	c3	= S.simorder[1]

	if (trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", S.iter)

		if (trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	S.params	= S.simplex[c1,]
	S.value	= S.simvals[c1]
	opt__log_value(S)

	ptol	= mreldif(S.simplex[c1,],S.simplex[c3,])
	vtol	= reldif(S.simvals[c1], S.simvals[c3])
	S.converged =	(ptol < S.ptol) | (vtol < S.vtol)
	if (S.converged) {
		if (ptol < vtol) {
			S.convtol	= ptol
			S.convtol_id	= "ptol"
		}
		else {
			S.convtol	= vtol
			S.convtol_id	= "vtol"
		}
	}
	else	S.convtol = `OPT_convtol_default'
	return(0)
}

/*STATIC*/ `Errcode' opt__step_nr_hybrid(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	ptol, vtol, nrtol
	real scalar	v0, v00
	real scalar	dim, k
	     scalar	trace
	real rowvector	p0, p00, gx, delta
	real matrix	invH, X, Xk, L
	pragma unset invH
	pragma unset X
	pragma unset L

	ptol		= S.ptol
	vtol		= S.vtol
	nrtol		= S.nrtol
	v00 = v0	= S.value
	p00 = p0	= S.params

	trace		= S.trace

	invH = invsym(-S.H)
	S.concave = opt__check_concave(S.H,invH)

	// report the value from the original parameters before we compute
	// the value at the next step
	opt__print_log(S)

	if (S.concave) {
		delta = S.gradient*invH
		S.params = p0 + delta
		if (trace >= `OPT_trace_step') {
			opt__print_vector(
				delta,
				"Step",
				trace >= `OPT_trace_params',
				S.minimize
			)
			displayas("txt")
			printf("Parameters + step -> new parameters\n")
			if (trace >= `OPT_trace_params') {
				S.params
				printf("\n")
			}
		}
	}
	else {
		symeigensystem(-S.H, X, L)
		if (trace >= `OPT_trace_step') {
			displayas("txt")
			printf(
	"Dividing space into concave/nonconcave regions based on" +
	" eigenvalues\nof negative Hessian\n")
			if (trace > `OPT_trace_step') {
				printf(":\n")
				L
				printf("\n")
			}
		}
		gx	= S.gradient*X
		dim	= cols(L)
		if (L[1] > 0 & dim >=2) {
			// work with eigenvectors associated with
			// reasonably large eigenvalues
			k = min((sum(L:>=S.eigen*L[1]),dim-1) )
			Xk	= X[|1,1 \ dim,k|]
			delta	= gx[|1,1 \ 1,k|] *
				invsym(diag(L[|1,1 \ 1,k|]))*Xk'
			S.params = p0 + delta
			if (trace >= `OPT_trace_step') {
				opt__print_vector(
					delta,
	sprintf("Concave-space (dim = {res:%f}) Newton-Raphson step", k),
					trace >= `OPT_trace_params',
					S.minimize
				)
				displayas("txt")
				printf("Parameters + step -> new parameters\n")
				if (trace >= `OPT_trace_params') {
					S.params
					printf("\n")
				}
			}
			ec = opt__eval(S,0)
			if (ec) return(ec)
			if (missing(S.value) | S.value < v0) {
				if (trace >= `OPT_trace_step') {
					opt__print_value(
						.,
						S.value_id,
						S.value,
						S.minimize,
						"(initial step bad)",
						1
					)
				}
				ec = opt__steputil_backward(S,v0,p0,60)
				if (ec) return(ec)
				v0	= S.value
				p0	= S.params
			}
			else {
				if (trace >= `OPT_trace_step') {
					opt__print_value(
						.,
						S.value_id,
						S.value,
						S.minimize,
						"(initial step good)",
						1
					)
				}
				ec = opt__steputil_forward(S,delta,1/8,2)
				if (ec) return(ec)
				v0	= S.value
				p0	= S.params
			}
		}
		else	k = 0
		// work with eigenvectors associated with small or
		// negative eigenvalues
		k++
		delta	= gx[|1,k \ 1,dim|]
		Xk	= X[|1,k \ dim,dim|]
		S.value = v00
		opt__steputil_size(S, delta, delta)
		S.value = v0
		delta	= delta*Xk'
		// update parameters
		S.params = p0 + delta
		if (trace >= `OPT_trace_step') {
			opt__print_vector(
				delta,
	sprintf("\nNonconcave-space (dim = {res:%f}) steepest-ascent step",
					dim-k+1),
				trace >= `OPT_trace_params',
				S.minimize
			)
			displayas("txt")
			printf("Parameters + step -> new parameters\n")
			if (trace >= `OPT_trace_params') {
				S.params
				printf("\n")
			}
		}
	}

	// evaluate function at new parameters
	ec = opt__eval(S,0)
	if (ec) return(ec)

	if (missing(S.value) | S.value < v0) {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step bad)",
				1
			)
		}
		ec = opt__steputil_backward(S,v0,p0,(S.concave? 60 : 10))
		if (ec) return(ec)
		if (!S.concave) {
			if (missing(S.value) | S.value < v0) {
				S.params = p0
			}
		}
	}
	else {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step good)",
				1
			)
		}
		if (S.concave) {
			ec = opt__steputil_forward(S,delta,1/8,2)
		}
		else {
			ec = opt__steputil_forward(S,delta,1,2)
		}
		if (ec) return(ec)
		S.backup = `OPT_backup_default'
	}

	if (trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", S.iter)

		if (trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	// evaluate function and derivatives at new parameters
	ec = opt__eval(S,2)
	if (ec) return(ec)
	if (missing(S.value)) {
		S.errorcode = `Errcode_discon'
		S.errortext = `Errtext_discon'
		opt__errorhandler(S)
		return(S.errorcode)
	}

	// check for convergence
	if ((mreldif(S.params, p00) < ptol)
	 |  ( reldif(S.value,  v00) < vtol)) {
		if (S.concave) {
			delta	= S.gradient*invsym(-S.H)
			if (missing(delta)) {
				S.errorcode = `Errcode_H_notdefinite'
				if (S.minimize) {
					S.errortext = `Errtext_H_notPSD'
				}
				else {
					S.errortext = `Errtext_H_notNSD'
				}
				opt__errorhandler(S)
				return(S.errorcode)
			}
			S.convtol	= abs(S.gradient*delta')
			S.converged	= S.convtol < nrtol
		}
		else	S.convtol = `OPT_convtol_default'
	}

	// log value and step iteration count
	opt__log_value(S)

	return(0)
}

/*STATIC*/ `Errcode' opt__step_nr_m_marquardt(`OptStruct' S)
{
	`Errcode'	ec
	real scalar	ptol, vtol, nrtol
	real scalar	v0
	real scalar	ok, adj, tr0
	     scalar	trace
	real rowvector	p0, delta
	real matrix	invH
	pragma unset invH

	ptol		= S.ptol
	vtol		= S.vtol
	nrtol		= S.nrtol
	v0		= S.value
	p0		= S.params

	trace		= S.trace

	invH = invsym(-S.H)
	S.concave = opt__check_concave(S.H,invH)

	// report the value from the original parameters before we compute
	// the value at the next step
	opt__print_log(S)

	if (S.concave) {
		delta = S.gradient*invH
		S.params = p0 + delta
	}
	else {
		ok = 0
		adj = 1.1
		tr0 = trace(invH)
		while (!ok) {
			S.H = S.H - adj*abs(diag(diagonal(S.H)))
			invH	= invsym(-S.H)
			ok	= opt__check_concave(S.H,invH)
			adj	= 2*adj
		}
		adj = tr0/trace(invH)
		if (adj > 0) {
			invH	= adj*invH
		}
		delta = S.gradient*invH
		if (missing(delta)) {
			S.errorcode = `Errcode_H_notdefinite'
			if (S.minimize) {
				S.errortext = `Errtext_H_notPSD'
			}
			else {
				S.errortext = `Errtext_H_notNSD'
			}
			opt__errorhandler(S)
			return(S.errorcode)
		}
		// not concave
		opt__steputil_size(S, S.gradient, delta)
		// update parameters
		S.params = p0 + delta
	}

	if (trace >= `OPT_trace_step') {
		opt__print_vector(
			delta,
			"Step",
			trace >= `OPT_trace_params',
			S.minimize
		)
		displayas("txt")
		printf("Parameters + step -> new parameters\n")
		if (trace >= `OPT_trace_params') {
			S.params
			printf("\n")
		}
	}

	// evaluate function at new parameters
	ec = opt__eval(S,0)
	if (ec) return(ec)

	if (missing(S.value) | S.value < v0) {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step bad)",
				1
			)
		}
		ec = opt__steputil_backward(S,v0,p0,60)
		if (ec) return(ec)
	}
	else {
		if (trace >= `OPT_trace_step') {
			opt__print_value(
				.,
				S.value_id,
				S.value,
				S.minimize,
				"(initial step good)",
				1
			)
		}
		if (S.concave) {
			ec = opt__steputil_forward(S,delta,1/8,2)
		}
		else {
			ec = opt__steputil_forward(S,delta,1,2)
		}
		if (ec) return(ec)
		S.backup = `OPT_backup_default'
	}

	if (trace >= `OPT_trace_step') {
		displayas("txt")
		printf("{hline}\n")
		printf("Iteration %f:\n", S.iter)

		if (trace >= `OPT_trace_params') {
			printf("Parameter vector:\n")
			S.params
			printf("\n")
		}
	}

	// evaluate function and derivatives at new parameters
	ec = opt__eval(S,2)
	if (ec) return(ec)

	if (missing(S.value)) {
		S.errorcode = `Errcode_discon'
		S.errortext = `Errtext_discon'
		opt__errorhandler(S)
		return(S.errorcode)
	}

	// check for convergence
	if ((mreldif(S.params, p0) < ptol)
	 |  ( reldif(S.value,  v0) < vtol)) {
		if (S.concave) {
			delta	= S.gradient*invsym(-S.H)
			if (missing(delta)) {
				S.errorcode = `Errcode_H_notdefinite'
				if (S.minimize) {
					S.errortext = `Errtext_H_notPSD'
				}
				else {
					S.errortext = `Errtext_H_notNSD'
				}
				opt__errorhandler(S)
				return(S.errorcode)
			}
			S.convtol	= abs(S.gradient*delta')
			S.converged	= S.convtol < nrtol
		}
		else	S.convtol = `OPT_convtol_default'
	}

	// log value and step iteration count
	opt__log_value(S)

	return(0)
}

/*STATIC*/ `Errcode' opt__steputil_backward(
	`OptStruct'		S,
	real	scalar		v0,		// previous value
	real	rowvector	p0,		// previous parameters
	real	scalar		maxback
)
{
	`Errcode'	ec
	real scalar	iback, size
	size	= norm(S.params-p0)
	iback	= 0
	while (iback < maxback & (missing(S.value) | S.value < v0)) {
		iback++
		S.params = (S.params + p0)/2
		size	= size/2
		// evaluate function at new parameters
		ec = opt__eval(S,0)
		if (ec) return(ec)
		if (S.trace >= `OPT_trace_step') {
			opt__steputil_log(S,
				iback,"Reducing step size",size)
		}
	}
	if (iback >= maxback) S.params = p0
	S.backup = iback
	return(0)
}

/*STATIC*/ `Errcode' opt__steputil_forward(
	`OptStruct' S,
	real rowvector	delta,		// step direction
	real scalar	start,		// initial stepsize
	real scalar	inc		// stepsize increase for next step
)
{
	`Errcode'	ec
	real scalar	istep
	real scalar	v1
	real rowvector	p1

	istep	= 0
	delta	= start*delta
	do {
		v1	= S.value
		p1	= S.params
		S.params = p1 + delta
		// evaluate function at new parameters
		ec = opt__eval(S,0)
		if (ec) return(ec)
		if (S.trace >= `OPT_trace_step') {
			opt__steputil_log(
				S,
				++istep,
				"Stepping forward",
				norm(delta))
		}
		delta = inc*delta
	} while (S.value > v1 & !missing(S.value))
	if (S.trace >= `OPT_trace_step') {
		printf("{right:(ignoring last step)}\n")
	}
	S.value		= v1
	S.params	= p1
	S.backup	= 0
	return(0)
}

/*STATIC*/ void opt__steputil_log(
	`OptStruct' S,
	real	scalar	istep,			// step counter
	string	scalar	message,		// type of stepping performed
	real	scalar	size			// length of the step
)
{
	if (S.trace < `OPT_trace_step') return

	printf("(%1.0f) %s (step length = {res:%9.0g})",
		istep, message, size)
	if (S.trace > `OPT_trace_step'){
		printf("\nNew parameter vector:\n")
		S.params
		printf("\n")
	}
	opt__print_value(
		.,
		S.value_id,
		S.value,
		S.minimize,
		"",
		1
	)
}

/*STATIC*/ void opt__steputil_size(
	`OptStruct' S,
	real rowvector grad,
	real rowvector delta)
{
	real scalar lower, upper, eps
	real scalar i, v0, v1, c

	upper = 1e-2
	lower = 1e-8

	if (S.iter <= 2) {
		eps = upper
	}
	else {
		i = mod(S.iter-2, 20) + 1
		v0 = S.log[i]
		i = mod(S.iter-1, 20) + 1
		v1 = S.log[i]
		eps = min( (upper, max( (lower,0.1*(v1-v0)/abs(v0)) ) ) )
	}
	c = eps*abs(S.value)/(grad*delta')
	if (missing(c)) {
		c = 2
	}
	delta = c * delta
}

/*STATIC*/ function opt__tracelevel_num(`OptStruct' S, |real scalar trace)
{
	if (args() == 2) {
		opt__set_int(S.trace, trace, 0, `OPT_trace_max')
		S.trace = trace
	}
	else	return(S.trace)
}

/*STATIC*/ function opt__tracelevel_str(`OptStruct' S, |string scalar trace)
{
	if (args() == 2) {
		real scalar len
		len = strlen(trace)
		if (trace == substr(`OPT_trace0',1,max((1,len)))) {
			S.trace = 0
		}
		else if (trace == substr(`OPT_trace1',1,max((1,len)))) {
			S.trace = 1
		}
		else if (trace == substr(`OPT_trace2',1,max((2,len)))) {
			S.trace = 2
		}
		else if (trace == substr(`OPT_trace3',1,max((1,len)))) {
			S.trace = 3
		}
		else if (trace == substr(`OPT_trace4',1,max((1,len)))) {
			S.trace = 4
		}
		else if (trace == substr(`OPT_trace5',1,max((1,len)))) {
			S.trace = 5
		}
		else if (trace == substr(`OPT_trace6',1,max((1,len)))) {
			S.trace = 6
		}
		else {
			_error("invalid trace level")
			/*NOTREACHED*/
		}
	}
	else {
		if (S.trace == 0) {
			return(`OPT_trace0')
		}
		if (S.trace == 1) {
			return(`OPT_trace1')
		}
		if (S.trace == 2) {
			return(`OPT_trace2')
		}
		if (S.trace == 3) {
			return(`OPT_trace3')
		}
		if (S.trace == 4) {
			return(`OPT_trace4')
		}
		if (S.trace == 5) {
			return(`OPT_trace5')
		}
		if (S.trace == 6) {
			return(`OPT_trace6')
		}
	}
}

/*STATIC*/ `Errcode' opt__v0_h_2(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real colvector	fph,			// f(x+h)
	real colvector	fmh,			// f(x-h)
	real rowvector	g0,
	real matrix	H0
)
{
	real scalar	goal0, goal1, mgoal, mingoal, iter, itmax
	real scalar	newscale, scale1, scale2, bestscale
	real scalar	dv, dv1, dv2, bestdv
	real scalar	v0
	real rowvector	g0_v
	real rowvector	p_alt
	pragma unset v0
	pragma unset g0_v

	goal0	= 1e-8
	goal1	= 1e-7
	mingoal	= 1e-12
	itmax	= 40

	goal0	= (abs(S.value)+goal0)*goal0
	goal1	= (abs(S.value)+goal1)*goal1
	mgoal	= (goal0+goal1)/2
	mingoal	= (abs(S.value)+mingoal)*mingoal

	p_alt	= S.params

	dv	= (abs(S.value - quadcross(S.weights, fph)) +
		   abs(S.value - quadcross(S.weights, fmh)))/2
	bestdv	= dv
	bestscale	= scale
	scale1	= 0
	dv1	= 0
	// use interpolation to get the scaled delta to within our goals
	for (iter=1; iter<=itmax & (dv<goal0 | dv>goal1); iter++) {
		if (dv < mingoal) {
			newscale = 2*scale
		}
		else if (iter==1) {
			// linear interpolation
			opt__ipolate_linear(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1)
		}
		else {
			// quadratic interpolation
			pragma unset scale2
			pragma unset dv2
			opt__ipolate_quadratic(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1,
				scale2,		dv2)
		}

		if (missing(newscale)
		 | newscale <= 0
		 | (dv > goal1 & newscale > scale)
		 | (dv < goal0 & newscale < scale)) {
			if (dv < goal0) {
				newscale	= 2*scale
			}
			else	newscale	= scale/2
		}

		scale2	= scale1
		dv2	= dv1
		scale1	= scale
		dv1	= dv

		scale	= newscale

		p_alt[i]= S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,S.minimize,
			0,p_alt,S.weights,v0,g0,H0,fmh,g0_v)
		if (!missing(fmh)) {
			p_alt[i]= S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,v0,g0,H0,fph,g0_v)
		}
		if (missing(fmh) | missing(fph)) {
			if (bestdv >= mingoal) {
				scale	= bestscale
				p_alt[i]= S.params[i] - floatround(h*scale)
				(*S.calluser)(S.user,S.arglist,
					S.minimize,
					0,p_alt,S.weights,v0,g0,H0,fmh,g0_v)
				p_alt[i]= S.params[i] + floatround(h*scale)
				(*S.calluser)(S.user,S.arglist,
					S.minimize,
					0,p_alt,S.weights,v0,g0,H0,fph,g0_v)

				displayas("txt")
				printf(
	"numerical derivatives are approximate\nnearby values are missing\n"
				)
				return(0)
			}
			S.errorcode = `Errcode_discon_miss'
			S.errortext = `Errtext_discon_miss'
			opt__errorhandler(S)
			return(S.errorcode)
		}

		dv = (abs(S.value - quadcross(S.weights, fph)) +
		      abs(S.value - quadcross(S.weights, fmh)))/2

		if (dv > 1.1*bestdv
		 | (dv >= 0.9*bestdv & scale < bestscale)) {
			bestdv		= dv
			bestscale	= scale
		}
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		if (bestdv >= mingoal) {
			// go with the best value
			scale	= bestscale
			p_alt[i]= S.params[i] - floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,v0,g0,H0,fmh,g0_v)
			p_alt[i]= S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,v0,g0,H0,fph,g0_v)

			displayas("txt")
			printf(
	"numerical derivatives are approximate\n" +
	"flat or discontinuous region encountered\n"
			)
		}
		else {
			S.errorcode = `Errcode_discon_flat'
			S.errortext = `Errtext_discon_flat'
			opt__errorhandler(S)
			return(S.errorcode)
		}
	}
	return(0)
}

/*STATIC*/ `Errcode' opt__v0_h(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real colvector	fph,
	real colvector	fmh,
	real rowvector	g0,
	real matrix	H0
)
{
	`Errcode'	ec
	real scalar	scale

	h = 1e-3
	h = (abs(S.params[i])+h)*h
	// get the numerical derivative scale for the ith parameter
	scale = S.scale[i]

	ec = opt__v0_h_size(S,i,h,scale,fph,fmh,g0,H0)
	if (ec) return(ec)

	// save the new numerical derivative scale for the ith parameter
	S.scale[i] = scale

	// return the value of the perturbation for taking a numerical
	// derivative with respect to the ith parameter
	h = floatround(h*scale)
	return(0)
}

/*STATIC*/ `Errcode' opt__v0_h_missing(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real colvector	fph,			// f(x+h)
	real colvector	fmh,			// f(x-h)
	real rowvector	g0,
	real matrix	H0
)
{
	real colvector	fm, fp
	real scalar	iter, itmax
	real rowvector	p_alt
	real scalar	v0
	real matrix	g0_v
	pragma unset fm
	pragma unset fp
	pragma unset v0
	pragma unset g0_v

	itmax	= 50

	p_alt = S.params
	fp	= fph
	fm	= fmh
	for (iter=1; iter<=itmax & (missing(fm) | missing(fp)); iter++) {
		scale = scale/2
		p_alt[i] = S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,
			S.minimize,
			0,p_alt,S.weights,v0,g0,H0,fm,g0_v)
		if (!missing(fm)) {
			p_alt[i] = S.params[i] + floatround(h*scale)
			(*S.calluser)(S.user,S.arglist,
				S.minimize,
				0,p_alt,S.weights,v0,g0,H0,fp,g0_v)
		}
	}
	if (missing(fm) | missing(fp)) {
		S.errorcode = `Errcode_discon_miss'
		S.errortext = `Errtext_discon_miss'
		opt__errorhandler(S)
		return(S.errorcode)
	}
	fmh = fm
	fph = fp
	return(opt__v0_h_2(S,i,h,scale,fph,fmh,g0,H0))
}

/*STATIC*/ `Errcode' opt__v0_h_size(
	`OptStruct' S,
	real scalar	i,
	real scalar	h,
	real scalar	scale,
	real colvector	fph,			// f(x+h)
	real colvector	fmh,			// f(x-h)
	real matrix	g0,
	real matrix	H0
)
{
	`Errcode'	ec
	real scalar	goal0, goal1, mgoal, mingoal, iter, itmax
	real colvector	fm0
	real scalar	scale0, newscale, scale1, scale2
	real scalar	dv, dv1, dv2
	real rowvector	p_alt
	real scalar	v0
	real matrix	g0_v
	pragma unset v0
	pragma unset g0_v

	if (S.iter == 0) {
		goal0	= 1e-4
		goal1	= 1e-3
	}
	else {
		goal0	= 1e-8
		goal1	= 1e-7
	}
	mingoal	= 1e-10
	itmax	= 20

	goal0	= (abs(S.value)+goal0)*goal0
	goal1	= (abs(S.value)+goal1)*goal1
	mgoal	= (goal0+goal1)/2
	mingoal	= (abs(S.value)+mingoal)*mingoal

	p_alt	= S.params

	p_alt[i]= S.params[i] - floatround(h*scale)
	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,p_alt,S.weights,v0,g0,H0,fmh,g0_v)
	if (missing(fmh)) {
		return(opt__v0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
	}

	// Save initial values, we may have to restore them later
	scale0	= scale
	fm0	= fmh

	dv	= abs(S.value-quadcross(S.weights, fmh))
	scale1	= 0
	dv1	= 0
	// use interpolation to get the scaled delta to within our goals
	for (iter=1; iter<=itmax & (dv<goal0 | dv>goal1); iter++) {
		if (dv < mingoal) {
			newscale = 2*scale
		}
		else if (iter==1) {
			// linear interpolation
			opt__ipolate_linear(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1)
		}
		else {
			// quadratic interpolation
			pragma unset scale2
			pragma unset dv2
			opt__ipolate_quadratic(
				newscale,	mgoal,
				scale,		dv,
				scale1,		dv1,
				scale2,		dv2)
		}

		if (missing(newscale)
		 | newscale <= 0
		 | (dv > goal1 & newscale > scale)
		 | (dv < goal0 & newscale < scale)) {
			if (dv < goal0) {
				newscale	= 2*scale
			}
			else	newscale	= scale/2
		}

		scale2	= scale1
		dv2	= dv1
		scale1	= scale
		dv1	= dv

		scale	= newscale

		p_alt[i]= S.params[i] - floatround(h*scale)
		(*S.calluser)(S.user,S.arglist,S.minimize,
			0,p_alt,S.weights,v0,g0,H0,fmh,g0_v)
		if (missing(fmh)) {
			return(opt__v0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
		}
		dv	= abs(S.value-quadcross(S.weights, fmh))
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		scale	= scale0	// go back to the initial values
		fmh	= fm0		// guaranteed to be nonmissing
	}

	p_alt[i]	= S.params[i] + floatround(h*scale)
	(*S.calluser)(S.user,S.arglist,S.minimize,
		0,p_alt,S.weights,v0,g0,H0,fph,g0_v)
	if (missing(fph)) {
		return(opt__v0_h_missing(S,i,h,scale,fph,fmh,g0,H0))
	}

	if (dv < goal0 | dv > goal1) {	// we did not meet the goal
		// redo the stepsize adjustment looking at both sides
		// starting values are guaranteed not to be missing
		return(opt__v0_h_2(S,i,h,scale,fph,fmh,g0,H0))
	}
	return(0)
}

/*STATIC*/ `Errcode' opt__validate(`OptStruct' S)
{
	real	scalar	k
	string	scalar	name

	S.valid	= 0

	// check user's evaluator
	if (S.user == `OPT_user_default') {
		_error("evaluator function required")
		/*NOTREACHED*/
	}

	// check starting values
	if (S.p0 == `OPT_p0_default') {
		_error("starting values for parameters required")
		/*NOTREACHED*/
	}
	if (missing(S.p0)) {
		_error("missing parameter values not allowed")
		/*NOTREACHED*/
	}
	S.params = S.p0

	// initialize work items based on dimension of the parameter vector
	S.dim	= cols(S.params)
	if (S.dim < 1) {
		_error("invalid starting values")
		/*NOTREACHED*/
	}
	S.gradient	= J(1,S.dim,.)
	S.H		= J(S.dim,S.dim,.)
	S.scale		= J(1,S.dim,`OPT_scale_default')

	// find the routine that calls the user's evaluator
	if (S.evaltype <= `OPT_evaltype_d2debug') {
		name	= sprintf("opt__calluser%f_d()",S.nargs)
	}
	else {
		name	= sprintf("opt__calluser%f_v()",S.nargs)
	}
	S.calluser = findexternal(name)
	if (S.calluser == `OPT_calluser_default') {
		_error(sprintf("internal routine '%s' not found", name))
		/*NOTREACHED*/
	}

	// find the internal looping routine for the technique
	if (S.loop == `OPT_loop_default') {
		optimize_init_technique(S, `OPT_technique_default')
	}

	if (S.technique == "nm") {
		if (S.simplex_delta == `OPT_simplex_delta_default') {
			_error("simplex delta required")
			/*NOTREACHED*/
		}
		k = cols(S.simplex_delta)
		if (k > 1 & k != S.dim) {
			_error(
			"simplex delta not conformable with parameter vector")
			/*NOTREACHED*/
		}
		if (min(S.simplex_delta) < 10*S.ptol) {
			_error(
		"simplex delta value too small (must be greater than 10*ptol)")
			/*NOTREACHED*/
		}
		S.convtol_id	= ""
	}
	else	S.convtol_id	= "nrtol"

	// find the internal routine that handles calling the user's evaluator
	// and other things like taking numerical derivatives
	name	= sprintf("opt__eval_%s_%s()",
			optimize_init_technique(S),
			optimize_init_evaluatortype(S))
	S.eval = findexternal(name)
	if (S.eval == `OPT_eval_default') {
		// check for invalid use of the d?debug/v?debug evaltypes
		if (S.technique != `OPT_technique_nr') {
			if (S.evaltype == `OPT_evaltype_d1debug') {
				_error("d1debug requires the nr technique")
				/*NOTREACHED*/
			}
			if (S.evaltype == `OPT_evaltype_d2debug') {
				_error("d2debug requires the nr technique")
				/*NOTREACHED*/
			}
			if (S.evaltype == `OPT_evaltype_v1debug') {
				_error("v1debug requires the nr technique")
				/*NOTREACHED*/
			}
			if (S.evaltype == `OPT_evaltype_v2debug') {
				_error("v2debug requires the nr technique")
				/*NOTREACHED*/
			}
		}
		_error(sprintf("internal routine '%s' not found", name))
		/*NOTREACHED*/
	}

	// find the internal routine for stepping
	name = sprintf("opt__step_%s_%s()",
		optimize_init_technique(S),
		subinstr(optimize_init_singularHmethod(S),
			"-", "_"))
	S.stepper = findexternal(name)
	if (S.stepper == `OPT_stepper_default') {
		_error(sprintf("internal routine '%s' not found", name))
		/*NOTREACHED*/
	}

	// linear constraints
	if (S.constraints != `OPT_constraints_default') {
		real scalar nc, p, kp1, pp1
		real matrix R, z, X, L
		pragma unset X
		pragma unset L
		nc	= rows(S.constraints)
		p	= cols(S.constraints) - 1
		if (p != cols(S.params) | nc >= p) {
			// conformability error
			_error(3200)
			/*NOTREACHED*/
		}
		k	= p - nc
		kp1	= k + 1
		pp1	= p + 1
		R	= S.constraints[|1,1 \ nc,p|]
		z	= I(p) - R'*invsym(R*R')*R
		_symeigensystem(z, X, L)
		if (L[kp1] > .5) {
			S.errorcode = `Errcode_badconstr'
			S.errortext = `Errtext_badconstr'
			opt__errorhandler(S)
			return(S.errorcode)
		}
		S.T	= X[|1,1 \ p,k|]
		L	= X[|1,kp1 \ p,p|]
		S.a	= S.constraints[|1,pp1 \ nc,pp1|]'*pinv(L'R')*L'
		S.hasCns = 1
	}
	else	S.hasCns = 0

	S.converged	= `OPT_converged_default'
	S.errorcode	= `OPT_errorcode_default'
	S.errortext	= `OPT_errortext_default'

	S.valid	= 1
	return(0)
}

/*STATIC*/ function opt__which_num(`OptStruct' S, |real scalar minimize)
{
	if (args() == 2) {
		opt__set_int(S.minimize,minimize,0,`OPT_which_last')
	}
	else	return(S.minimize)
}

/*STATIC*/ function opt__which_str(`OptStruct' S, |string scalar which)
{
	if (args() == 2) {
		if (which == `OPT_which0') {
			S.minimize	= 0
		}
		else if (which == `OPT_which1') {
			S.minimize	= 1
		}
		else {
			_error("invalid which")
			/*NOTREACHED*/
		}
	}
	else {
		if (S.minimize == 0) {
			return(`OPT_which0')
		}
		if (S.minimize == 1) {
			return(`OPT_which1')
		}
	}
}

/*STATIC*/ function opt__weights(
	`OptStruct' S,
	|	scalar		wtype,
	real	colvector	weights
)
{
	if (args() == 3) {
		if (missing(weights)) {
			_error("missing values not allowed in weights")
			/*NOTREACHED*/
		}
		opt__wtype(S, wtype)
		if (S.wtype) {
			if (S.wtype == `OPT_wtype_a') {
				S.weights = weights:/mean(weights)
			}
			else	S.weights = weights
		}
	}
	else if (args() == 2) {
		_error("invalid number of arguments")
		/*NOTREACHED*/
	}
	else {
		return(S.weights)
	}
}

/*STATIC*/ function opt__wtype(`OptStruct' S, |scalar wtype)
{
	if (args() == 1) {
		return(opt__wtype_str(S))
	}
	else if (isreal(wtype)) {
		opt__wtype_num(S, wtype)
	}
	else if (isstring(wtype)) {
		opt__wtype_str(S, wtype)
	}
	else {
		_error("invalid weight type")
		/*NOTREACHED*/
	}
}

/*STATIC*/ function opt__wtype_num(`OptStruct' S, |real scalar wtype)
{
	if (args() == 2) {
		opt__set_int(S.wtype,wtype,0,`OPT_wtype_max')
	}
	else	return(S.wtype)
}

/*STATIC*/ function opt__wtype_str(`OptStruct' S, |string scalar wtype)
{
	if (args() == 2) {
		real scalar	len
		len	= strlen(wtype)
		if (len == 0) {
			S.wtype	= 0
		}
		else if (wtype == substr("weight",1,len)) {
			S.wtype	= `OPT_wtype_f'
		}
		else if (wtype == substr(`OPT_wtype1',1,len)) {
			S.wtype	= 1
		}
		else if (wtype == substr(`OPT_wtype2',1,len)) {
			S.wtype	= 2
		}
		else if (wtype == substr(`OPT_wtype3',1,len)) {
			S.wtype	= 3
		}
		else if (wtype == substr(`OPT_wtype4',1,len)) {
			S.wtype	= 4
		}
		else {
			_error("invalid weight type")
			/*NOTREACHED*/
		}
	}
	else {
		if (S.wtype == 0) {
			return(`OPT_wtype0')
		}
		if (S.wtype == 1) {
			return(`OPT_wtype1')
		}
		if (S.wtype == 2) {
			return(`OPT_wtype2')
		}
		if (S.wtype == 3) {
			return(`OPT_wtype3')
		}
		if (S.wtype == 4) {
			return(`OPT_wtype4')
		}
	}
}

end
