{smcl}
{* *! version 2.1.1  05aug2008}{...}
{cmd:help random-number functions}
{hline}

{title:Title}

{p2colset 5 22 24 2}{...}
{p2col :{hi:[D] functions} {hline 2}}Functions{p_end}
{p2colreset}{...}


{title:Description}

{pstd}This is a quick reference on functions for generating pseudorandom 
variates.  For help on all functions, see {manhelp functions D}.  See
{manhelp generate D} for setting the random-number seed.


{title:Random-number functions}

    {cmd:runiform()}
{p2colset 8 22 26 2}{...}
{p2col: Range:}0 to nearly 1 (0 to 1 - 2^(-32)){p_end}
{p2col: Description:}returns uniform random variates.{p_end}

{p2col 8 22 22 2:}{cmd:runiform()} returns uniformly distributed random
variates on the interval [0,1).  {cmd:runiform()} takes no arguments, but the
parentheses must be typed.  {cmd:runiform()} can be seeded with the 
{cmd:set seed} command. (See {help matrix functions} for the related
{cmd:matuniform()} matrix function.){p_end}

{p2col 8 22 22 2:}To generate random variates over the interval
[{it:a},{it:b}), use {it:a}{cmd:+(}{it:b}{cmd:-}{it:a}{cmd:)*runiform()}.{p_end}

{p2col 8 22 22 2:}To generate random integers over [{it:a},{it:b}],
use {it:a}{cmd:+int((}{it:b}{cmd:-}{it:a}{cmd:+1)*runiform())}.{p_end}
{p2colreset}{...}

    {cmd:rbeta(}{it:a}{cmd:,}{it: b}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:a}:}0.05 to 1e5{p_end}
{p2col: Domain {it:b}:}0.15 to 1e5{p_end}
{p2col: Range:}0 to 1 (exclusive){p_end}
{p2col: Description:}returns beta({it:a},{it:b}) random variates, where
	{it:a} and {it:b} are the beta shape parameters. 

{p2col 8 22 22 2:}Besides the standard methodology for generating random 
	variates from a given distribution, {cmd:rbeta()} employs the 
	specialized algorithms of Johnk (Gentle 2003), 
	Atkinson and Wittaker (1976, 1979), Devroye (1986), and 
	Schmeiser and Babu (1980).
{p2colreset}{...}

    {cmd:rbinomial(}{it:n}{cmd:,}{it: p}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:n}:}1 to 1e11{p_end}
{p2col: Domain {it:p}:}1e-8 to 1-1e-8{p_end}
{p2col: Range:}0 to {it:n}{p_end}
{p2col: Description:}returns binomial({it:n},{it:p}) random variates, where
	{it:n} is the number of trials and {it:p}
	is the success probability.  

{p2col 8 22 22 2:}Besides the standard methodology for generating random 
	variates from a given distribution, {cmd:rbinomial()} employs the 
	specialized algorithms of Kachitvichyanukul (1982), 
	Kachitvichyanukul and Schmeiser (1988), and Kemp (1986).
{p2colreset}{...}

    {cmd:rchi2(}{it:df}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:df}:}2e-4 to 2e8{p_end}
{p2col: Range:}0 to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns chi-squared, with {it:df} degrees of 
	freedom, random variates.
{p2colreset}{...}

    {cmd:rgamma(}{it:a}{cmd:, }{it:b}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:a}:}1e-4 to 1e8{p_end}
{p2col: Domain {it:b}:}{cmd:c(smallestdouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Range:}0 to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns gamma({it:a},{it:b}) random variates, where {it:a} 
	is the gamma shape parameter and {it:b} is the scale parameter.

{p2col 8 22 22 2:}Methods for generating gamma variates are taken from 
	Ahrens and Dieter (1974), Best (1983), and Schmeiser and Lal (1980). 
{p2colreset}{...}

    {cmd:rhypergeometric(}{it:N}{cmd:,}{it: K}{cmd:,}{it: n}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:N}:}2 to 1e6{p_end}
{p2col: Domain {it:K}:}1 to {it:N-1}{p_end}
{p2col: Domain {it:n}:}1 to {it:N-1}{p_end}
{p2col: Range:}{cmd:max(}0{cmd:,}{it:n-N+K}{cmd:)} to 
{cmd:min(}{it:K,n}{cmd:)}{p_end}
{p2col: Description:}returns hypergeometric random variates.  The
	distribution parameters are integer valued, where 
	{it:N} is the population size, {it:K} is the 
	number of elements in the population that have the attribute of
	interest, and {it:n} is the sample size.  

{p2col 8 22 22 2:}Besides the standard methodology for generating random 
	variates from a given distribution, {cmd:rhypergeometric()} employs 
	the specialized algorithms of Kachitvichyanukul (1982) and 
	Kachitvichyanukul and Schmeiser (1985).
{p2colreset}{...}

    {cmd:rnbinomial(}{it:n}{cmd:,}{it: p}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:n}:}0.1 to 1e5{p_end}
{p2col: Domain {it:p}:}1e-4 to 1-1e-4{p_end}
{p2col: Range:}0 to 2^53-1{p_end}
{p2col: Description:}returns negative binomial random variates.
	If {it:n} is integer valued, {cmd:rnbinomial()} returns the number 
	of failures before the {it:n}th success, 
	where the probability of success on a single trial is {it:p}. 
	{it:n} can also be nonintegral.  
{p2colreset}{...}

    {cmd:rnormal()}
{p2colset 8 22 22 2}{...}
{p2col: Range:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns standard normal (Gaussian) random variates,
	i.e., variates from a normal distribution with a mean of 0 and a 
	standard deviation of 1.  

    {cmd:rnormal(}{it:m}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:m}:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Range:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns normal({it:m},1) (Gaussian) random variates,
	where {it:m} is the mean and the standard deviation is 1.
{p2colreset}{...}

    {cmd:rnormal(}{it:m}{cmd:,}{it: s}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:m}:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Domain {it:s}:}{cmd:c(smallestdouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Range:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns normal({it:m},{it:s}) (Gaussian) random variates,
	where {it:m} is the mean and {it:s} is the standard deviation.
{p2colreset}{...}

{p2col 8 22 22 2:}The methods for generating normal (Gaussian) random variates
are taken from Knuth (1998, 122-128); Marsaglia, MacLaren, and Bray (1964); and
Walker (1977).
{p2colreset}{...}

    {cmd:rpoisson(}{it:m}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:m}:}1e-6 to 1e11{p_end}
{p2col: Range:}0 to 2^53-1{p_end}
{p2col: Description:}returns Poisson({it:m}) random variates, where {it:m} 
	is the distribution mean.  

{p2col 8 22 22 2:}Poisson variates are generated using the probability 
	integral transform methods of Kemp and Kemp (1990, 1991), as well as 
	the method of Kachitvichyanukul (1982).
{p2colreset}{...}

    {cmd:rt(}{it:df}{cmd:)}
{p2colset 8 22 22 2}{...}
{p2col: Domain {it:df}:}1 to 2^53-1{p_end}
{p2col: Range:}{cmd:c(mindouble)} to {cmd:c(maxdouble)}{p_end}
{p2col: Description:}returns t({it:df}) random variates, where {it:df}
	is the degrees of freedom.  

{p2col 8 22 22 2:}Student's t variates are generated using the method 
	of Kinderman and Monahan (1977, 1980).
{p2colreset}{...}


{title:References}

{phang}
Ahrens, J. H., and U. Dieter.  1974.  Computer methods for sampling from gamma,
beta, Poisson, and binomial distributions.  {it:Computing} 12: 223-246.

{phang}
Atkinson, A. C., and J. Whittaker.  1976.  A switching algorithm for the 
generation of beta random variables with at least one parameter less
than 1.  {it:Journal of the Royal Statistical Society, Series A (General)}
139: 462-467.

{phang}
------. 1979. Algorithm AS 134: The generation of beta random
variables with one parameter greater than and one parameter less 
than 1.  {it:Applied Statistics} 28: 90-93.

{phang}
Best, D. J.  1983.  A note on gamma variate generators with shape parameters
less than unity.  {it:Computing} 30: 185-188.

{phang}
Devroye, L.  1986.  {it:Non-uniform Random Variate Generation}.
New York: Springer.

{phang}
Gentle, J. E.  2003.  {it:Random Number Generation and Monte Carlo Methods}. 
2nd ed.  New York: Springer.

{phang}
Kachitvichyanukul, V.  1982.
{it:Computer Generation of Poisson, Binomial, and Hypergeometric Random}
{it:Variables}.  PhD thesis.  Purdue University.

{phang}
Kachitvichyanukul, V., and B. Schmeiser.  1985.  Computer generation
of hypergeometric random variates.
{it:Journal of Statistical Computation and Simulation} 22: 127-145.

{phang}
------.  1988.  Binomial random variate generation.
{it:Communications of the ACM} 31: 216-222.

{phang} 
Kemp, C. D.  1986.  A modal method for generating binomial variates.  
{it:Communications in Statistics: Theory and Methods} 15: 805-813.

{phang} 
Kemp, A. W., and C. D. Kemp.  1990.  A composition-search algorithm for 
low-parameter Poisson generation.
{it:Journal of Statistical Computation and Simulation} 35: 239-244.

{phang} 
Kemp, C. D., and A. W. Kemp.  1991.  Poisson random variate generation. 
{it:Applied Statistics} 40: 143-158.

{phang}
Kinderman, A. J., and J. F. Monahan.  1977.  Computer generation of random
variables using the ratio of uniform deviates.
{it:ACM Transactions on Mathematical Software} 3: 257-260.

{phang}
------.  1980.  New methods for generating
Student's t and gamma variables.  {it:Computing} 25: 369-377.

{phang} 
Knuth, D.  1998.
{it:The Art of Computer Programming, Volume 2: Seminumerical Algorithms}.
3rd ed.  Reading, MA: Addison Wesley.

{phang} 
Marsaglia, G., M. D. MacLaren, and T. A. Bray.  1964.  A fast procedure for
generating normal random variables.  {it:Communications of the ACM} 7: 4-10.

{phang}
Schmeiser, B. W., and A. J. G. Babu.  1980.  Beta variate generation via 
exponential majorizing functions.  {it:Operations Research} 28: 917-926.

{phang}
Schmeiser, B. W., and R. Lal. 1980.
Squeeze methods for generating gamma variates.
{it:Journal of the American Statistical Association} 75: 679-682.

{phang}
Walker, A. J.  1977.  An efficient method for generating discrete random
variables with general distributions.
{it:ACM Transactions on Mathematical Software} 3: 253-256.


{title:Also see}

{psee}
Manual:  {bf:[D] functions}

{psee}
Online:  {manhelp drawnorm D}, {break}
{manhelp egen D}
{p_end}
