{smcl}
{* *! version 1.1.7  25feb2008}{...}
{cmd:help rotatemat}{right:dialog:  {bf:{dialog rotatemat}}}
{right:also see:  {helpb rotate}   }
{hline}

{title:Title}

{p2colset 5 23 25 2}{...}
{p2col:{hi:[MV] rotatemat} {hline 2}}Orthogonal and oblique rotations of a
Stata matrix{p_end}
{p2colreset}{...}


{title:Syntax}

{p 8 30 2}
{cmd:rotatemat} {it:matrix_L} [{cmd:,} {it:options}]
{p_end}

{synoptset 20 tabbed}{...}
{synopthdr}
{synoptline}
{syntab:Main}
{synopt:{opt ortho:gonal}}restrict to orthogonal rotations; the default,
	except with {cmd:promax()}{p_end}
{synopt:{opt obliq:ue}}allow oblique rotations{p_end}
{synopt:{it:rotation_methods}}rotation criterion{p_end}
{synopt:{opt norm:alize}}rotate Kaiser normalized matrix{p_end}

{syntab:Reporting}
{synopt:{opt for:mat(%fmt)}}display format for matrices;
	default is {cmd:format(%9.5f)}{p_end}
{synopt:{opt bl:anks(#)}}display numbers as blanks when
	|loadings| < {it:#}; default is {cmd:blanks(0)}{p_end}
{synopt:{opt nodisp:lay}}suppress all output except log and trace{p_end}
{synopt:{opt noload:ing}}suppress display of rotated loadings{p_end}
{synopt:{opt norota:tion}}suppress display of rotation matrix{p_end}
{synopt:{opt matn:ame(str)}}descriptive label of the matrix to be
	rotated{p_end}
{synopt:{opt coln:ames(str)}}descriptive name for columns of the matrix to be
	rotated{p_end}

{syntab:Opt options}
{synopt:{it:optimize_options}}control
	the maximization process; seldom used{p_end}
{synoptline}

INCLUDE help rotate_criteria_table


{title:Description}

{pstd}
{cmd:rotatemat} applies a linear transformation {bf:T} to the matrix
{it:matrix_L}, which we will call {bf:A}, so that the result
c({bf:A}*inv({bf:T}')) minimizes some criterion function c() over all matrices
{bf:T} in a class of feasible transformations.  Two classes are
supported: orthogonal (orthonormal) and oblique.  orthonormal rotations
comprise all orthonormal matrices {bf:T}, such that
{bf:T}'*{bf:T} = {bf:T}*{bf:T}' = {bf:I}; here {bf:A}*inv({bf:T}')
simplifies to {bf:A}*{bf:T}.  Oblique rotations are characterized by
diag({bf:T}'*{bf:T}) = {bf:1}.  A wide variety of criteria c() is available,
representing different ways to measure the "simplicity" of a matrix.  Most of
these criteria can be applied with both orthogonal and oblique rotations.

{pstd}
If you are interested in rotation after {cmd:factor}, {cmd:factormat},
{cmd:pca}, or {cmd:pcamat}, see
{manhelp factor_postestimation MV:factor postestimation}, 
{manhelp pca_postestimation MV:pca postestimation}, and the general
description of {cmd:rotate} as a postestimation facility in 
{bf:[MV] rotate}.

{pstd}
This entry describes the computation engine for orthogonal and oblique
transformations of Stata matrices.  This command may be used directly
on any Stata matrix.


{title:Options}

{dlgtab:Main}

{phang}{opt orthogonal}
specifies that an orthogonal rotation be applied.  This is the default.

{pmore}
See {it:{help rotate##criteria:Rotation criteria}} below for details on the
{it:rotation_methods} available with {opt orthogonal}.

{phang}{opt oblique}
specifies that an oblique rotation be applied.  This often yields more
interpretable factors with a simpler structure than obtained with an
orthogonal rotation.  In many applications (e.g., after {cmd:factor} and
{cmd:pca}) the factors before rotation are orthogonal (uncorrelated), whereas 
the oblique rotated factors are correlated.

{pmore}
See {it:{help rotate##criteria:Rotation criteria}} below for details on the
{it:rotation_methods} available with {opt oblique}.

{phang}{opt normalize}
requests that the rotation be applied to the Kaiser normalization of the matrix
{bf:A}, so that the rowwise sums of squares r equal 1.

{dlgtab:Reporting}

{phang}{opt format(%fmt)}
specifies the display format for matrices.  The default is
{cmd:format(%9.5f)}.

{phang}{opt blanks(#)}
specifies that small values of the rotated matrix -- i.e., those elements
of {bf:A} ({bf:T}')^{-1} that are less than {it:#} in absolute value --
are displayed as spaces.

{phang}{opt nodisplay}
suppresses all output except the log and trace.

{phang}{opt noloading}
suppresses the display of the rotated matrix.

{phang}{opt norotation}
suppresses the display of the optimal rotation matrix.

{phang}{opt matname(str)}
is a rarely used output option; it specifies a descriptive label of the matrix
to be rotated.

{phang}{opt colnames(str)}
is a rarely used output option; it specifies a descriptive name to refer to
the columns of the matrix to be rotated.  For instance,
{cmd:colnames(components)} specifies that the output labels the columns as
"components".  The default is "factors".

{dlgtab:Opt options}

{phang}
{it:optimize_options} control the iterative optimization process.  These
options are seldom used.

{phang2}{opt iterate(#)}
is a rarely used option; it specifies the maximum number of iterations.  The
default is {cmd:iterate(1000)}.

{phang2}{opt log}
specifies that an iteration log be displayed.

{phang2}{opt trace}
is a rarely used option; it specifies that the rotation be displayed at each
iteration.

{phang2}{opt tolerance(#)}
is one of three criteria for declaring convergence and is rarely used.  The
{cmd:tolerance()} convergence criterion is satisfied when the relative change
in the rotation matrix {bf:T} from one iteration to the next is less than or
equal to {it:#}.  The default is {cmd:tolerance(1e-6)}.

{phang2}{opt gtolerance(#)}
is one of three criteria for declaring convergence and is rarely used.  The
{cmd:gtolerance()} convergence criterion is satisfied when the Frobenius norm
of the gradient of the criterion function c() projected on the manifold of
orthogonal matrices or of normal matrices is less than or equal to {it:#}.
The default is {cmd:gtolerance(1e-6)}.

{phang2}{opt ltolerance(#)}
is one of three criteria for declaring convergence and is rarely used.  The
{cmd:ltolerance()} convergence criterion is satisfied when the relative change
in the minimization criterion c() from one iteration to the next is less than
or equal to {it:#}.  The default is {cmd:ltolerance(1e-6)}.

{phang2}{opt protect(#)}
requests that {it:#} optimizations with random starting values be performed
and that the best of the solutions be reported.  The output also indicates
whether all starting values converged to the same solution.  When specified
with a large number, such as {cmd:protect(50)}, this provides reasonable
assurance that the solution found is the global maximum and not just a local
maximum.  If {cmd:trace} is also specified, the rotation matrix and rotation
criterion value of each optimization will be reported.

{phang2}{opt maxstep(#)}
is a rarely used option; it specifies the maximum number of step-size
halvings.  The default is {cmd:maxstep(20)}.

{phang2}{opt init(matname)}
is a rarely used option; it specifies the initial rotation matrix.
{it:matname} should be square and regular (nonsingular) and have the same
number of columns as matrix {it:matrix_L} to be rotated.  It should be
orthogonal ({bf:T}'*{bf:T} = {bf:T}*{bf:T}' = {bf:I}) or normal
(diag({bf:T}'*{bf:T}) = {bf:1}), depending on whether orthogonal or oblique
rotations are performed.  {opt init()} cannot be combined with
{opt random}.  If neither {opt init()} nor {cmd:random} is specified, the
identity matrix is used as the initial rotation.

{phang2}{opt random}
is a rarely used option; it specifies that a random orthogonal or random
normal matrix is to be used as the initial rotation matrix.  {cmd:random}
cannot be combined with {opt init()}.  If neither {opt init()} nor
{opt random} is specified, the identity matrix is used as the initial
rotation.


{marker criteria}{...}
INCLUDE help rotate_criteria_description


{title:Remark on identification}

{pstd}
All supported criteria are invariant with respect to permutations of the
columns and change of signs of the columns.  {cmd:rotatemat} returns the
solution with positive column sums and with columns sorted by the L2 norm;
columns are ordered with respect to the L1 norm if the columns have the
same L2 norm.


{title:Examples}

    {cmd:. rotatemat L}
    {cmd:. rotatemat X, oblique}
    {cmd:. rotatemat L, entropy protect(20)}


{title:Saved results}

{pstd}
{cmd:rotatemat} saves the following in {cmd:r()}:

{synoptset 20 tabbed}{...}
{p2col 5 20 24 2: Scalars}{p_end}
{synopt:{cmd:r(f)}}criterion value{p_end}
{synopt:{cmd:r(iter)}}number of GPF iterations{p_end}
{synopt:{cmd:r(rc)}}return code{p_end}
{synopt:{cmd:r(nnconv)}}number of nonconvergent trials; {cmd:protect()} only{p_end}

{synoptset 20 tabbed}{...}
{p2col 5 20 24 2: Macros}{p_end}
{synopt:{cmd:r(cmd)}}{cmd:rotatemat}{p_end}
{synopt:{cmd:r(ctitle)}}descriptive label of rotation method{p_end}
{synopt:{cmd:r(ctitle12)}}version of {cmd:r(ctitle)} at most 12 characters
long{p_end}
{synopt:{cmd:r(criterion)}}criterion name (e.g., {cmd:oblimin}){p_end}
{synopt:{cmd:r(class)}}{cmd:orthogonal} or {cmd:oblique}{p_end}
{synopt:{cmd:r(normalization)}}{cmd:kaiser} or {cmd:none}{p_end}
{synopt:{cmd:r(carg)}}criterion argument{p_end}

{synoptset 20 tabbed}{...}
{p2col 5 20 24 2: Matrices}{p_end}
{synopt:{cmd:r(T)}}optimal transformation T{p_end}
{synopt:{cmd:r(AT)}}optimal AT = A(T')^-1{p_end}
{synopt:{cmd:r(fmin)}}minimums found; {cmd:protect()} only{p_end}
{p2colreset}{...}


{title:Also see}

{psee}
Manual:  {bf:[MV] rotatemat}
{p_end}

{psee}
Online:  {manhelp rotate MV};{break}
{manhelp procrustes MV}
{p_end}
