*! version 3.1.1  05nov2007
program svyset
	if _caller() < 9 {
		svyset_8 `0'
		exit
	}

	capture Display
	if !c(rc) {
		local oldset `"`r(settings)'"'
	}
	capture noisily SvySet `0'
	if c(rc) {
		local rc = c(rc)
		capture SvySet `oldset'
		exit `rc'
	}
end

program SvySet
	version 9
	// -eclass- is undocumented
	capture syntax [, ECLASS CLEAR ]
	if !c(rc) {
		if "`clear'" != "" {
			Clear
			Get
		}
		else	Display, `eclass'
		exit
	}
	if _N == 0 {
		di as err "no variables defined"
		exit 111
	}

	local commopts	VCE(passthru) MSE	///
			BRRweight(passthru)	///
			JKRweight(passthru)	///
			FAY(passthru)		///
			SINGLEunit(passthru)	///
			POSTstrata(passthru)	///
			POSTWeight(passthru)	///
			CLEAR1(passthru)	///
			CLEAR NOCLEAR
	_parse expand stage global : 0, common(`commopts')

	// [if] and [in] are not allowed
	if `"`global_if'`global_in'"' != "" {
		local 0 `"`global_if' `global_in'"'
		syntax [, NOOPTIONS ]
	}

	if `"`global_op'"' != "" {
		ParseGlobals `stage_n' , `global_op'
		if "`s(noclear)'" != "" & `stage_n' {
			local 0 `"`stage_1'"'
			capture syntax [pw iw/]
			if c(rc) {
				di as err ///
			"option noclear is not allowed with design options"
				exit 198
			}
			_svyset set `weight' `exp'
			local stage_n 0
		}
		local noclear `s(noclear)'
	}
	else {
		Clear
	}

	local ignore 0
	forval i = 1/`stage_n' {
		ParseStage `i' `stage_n' `ignore' `stage_`i''
		_svyset get fpc `i'
		if "`r(fpc`i')'" == "" & `stage_n' > `i' {
			di as txt "{p 0 7 2}" ///
"Note: stage `i' is sampled with replacement;" ///
" all further stages will be ignored{p_end}"
			local stage_n `i'
			local ignore 1
		}
	}
	if "`noclear'" == "" & `"`:char _dta[_svy_su1]'"' != "" {
		if `stage_n' == 0 {
			local stage_n 1
		}
		char _dta[_svy_stages] `stage_n'
	}
	if `stage_n' | `"`global_op'"' != "" {
		char _dta[_svy_version] 2
	}

	Display
end

program ParseGlobals, sclass
	gettoken stage_n 0 : 0, parse(" ,")
	if `stage_n' == 0 {
		// These are undocumented options that were allowed in a
		// single stage -svyset- in Stata 8.

		local oldopts PSU(passthru) STRata(passthru) FPC(passthru) SRS
	}
	syntax [,				///
		VCE(string)			///
		MSE				///
		BRRweight(varlist numeric)	///
		FAY(numlist max=1 >=0 <=2)	///
		JKRweight(string asis)		///
		POSTstrata(varname)		///
		POSTWeight(varname numeric)	///
		SINGLEunit(name)		///
		NOCLEAR				///
		CLEAR				///
		CLEAR1(namelist)		///
		`oldopts'			///
	]

	// NOTE: the -clear- option is accepted here, but it is ignored since
	// it is the default unless -noclear- or -clear()- are specified

	// initialize s(noclear)
	sreturn local noclear
	if "`noclear'" == "" & "`clear1'" == "" {
		Clear
	}
	else {
		if "`clear'" != "" {
			if "`clear1'" != "" {
				opts_exclusive "clear clear()"
			}
			if "`noclear'" != "" {
				opts_exclusive "clear noclear"
			}
		}
		Clear, `clear1' noclear
		if `stage_n' > 1 | `"`psu'`strata'`fpc'`srs'"' != "" {
			di as err ///
			"option noclear is not allowed with design options"
			exit 198
		}
		local noclear noclear
	}

	if `"`vce'`mse'"' != "" {
		_svyset set vce `vce', `mse'
		local defpsu _n
	}
	if `"`brrweight'"' != "" {
		_svyset set brrweight `brrweight'
	}
	if `"`fay'"' != "" {
		_svyset set fay `fay'
	}
	if `"`jkrweight'"' != "" {
		JKnifeVChars `jkrweight'
		_svyset set jkrweight `s(varlist)'
	}
	if `"`poststrata'`postweight'"' != "" {
		if "`postweight'" == "" {
			di as err ///
			"option poststrata() requires option postweight()"
			exit 198
		}
		if "`poststrata'" == "" {
			di as err ///
			"option postweight() requires option poststrata()"
			exit 198
		}
		_svyset set poststrata `poststrata'
		_svyset set postweight `postweight'
		local defpsu _n
	}
	if `"`singleunit'"' != "" {
		capture _svyset set singleunit `singleunit'
		if c(rc) {
			di as err "option singleunit() invalid"
			exit 198
		}
	}
	if `stage_n' == 0 & "`noclear'" == "" {
		if "`srs'" != "" {
			if "`strata'" != "" {
				opts_exclusive "strata() srs"
			}
			if "`psu'" != "" {
				opts_exclusive "psu() srs"
			}
			local mypsu _n
		}
		else if "`psu'" == "" {
			local mypsu `defpsu'
		}
		if "`mypsu'`psu'`strata'`fpc'" != "" {
			ParseStage 1 1 0 `mypsu', `psu' `strata' `fpc'
		}
	}
	sreturn local noclear `noclear'
end

program ParseStage, rclass
	gettoken i	0 : 0
	gettoken last	0 : 0
	gettoken ignore	0 : 0
	if `i' == 1 {
		local uspec `"[anything(name=su id="psuid")]"'
		local oldopts PSU(varname)
	}
	else {
		local uspec `"anything(name=su id="ssuid")"'
	}
	syntax `uspec' [pweight iweight/] [,		///
		STRata(varname)				///
		FPC(varname numeric)			///
		`oldopts'				///
	]
	if "`psu'" != "" {
		if "`su'" != "" & "`su'" != "`psu'" {
			di as err "option psu() invalid"
			exit 198
		}
		else	local su `psu'
	}
	else if "`su'" == "" {
		if "`weight'`strata'`fpc'" == "" {
			di as err "psuid required"
			exit 198
		}
		local su _n
	}
	if `"`su'"' == "_n" & `i' < `last' {
		di as err "invalid use of _n; " ///
"observations can only be sampled in the final stage"
		exit 198
	}
	// else `su' was given
	local wtype	`weight'
	local wvar	`"`exp'"'
	if `"`wtype'"' != "" {
		_svyset get wvar
		if `"`r(wvar)'"' != "" {
			di as err "sampling weights may only be specified once"
			exit 198
		}
		if `ignore'	exit
		_svyset set `wtype' `wvar'
	}
	if !inlist("`su'", "", "_n") {
		capture confirm variable `su'
		if c(rc) {
			local psu = cond(`i' == 1, "psu", "ssu")
			di as err `"`su' invalid `psu' variable name"'
			exit c(rc)
		}
	}
	if `ignore'	exit
	_svyset set su `i' `su'
	if "`strata'" != "" {
		_svyset set strata `i' `strata'
	}
	else	_svyset clear strata `i'
	if "`fpc'" != "" {
		_svyset set fpc `i' `fpc'
	}
	else	_svyset clear fpc `i'
end

program Clear
	syntax [, noCLEAR	///
		PWeight		///
		IWeight		///
		Weight		///
		vce		///
		mse		///
		BRRweight	///
		FAY		///
		JKRweight	///
		POSTstrata	///
		POSTweight	///
		SINGLEunit	///
	]

	// clear the current settings
	if "`clear'" == "" {
		quietly svyset_8, clear
		_svyset clear wtype
		_svyset clear vce
		_svyset clear brrweight
		_svyset clear fay
		_svyset clear jkrweight
		_svyset clear poststrata
		_svyset clear singleunit
		_svyset clear stages 1
		char _dta[_svy_version]
		exit
	}
	if "`pweight'`iweight'`weight'" != "" {
		_svyset clear wtype
	}
	if "`vce'" != "" {
		_svyset clear vce
	}
	if "`mse'" != "" {
		_svyset clear mse
	}
	if "`brrweight'" != "" {
		_svyset clear brrweight
	}
	if "`fay'" != "" {
		_svyset clear fay
	}
	if "`jkrweight'" != "" {
		_svyset clear jkrweight
	}
	if "`poststrata'`postweight'" != "" {
		_svyset clear poststrata
	}
	if "`singleunit'" != "" {
		_svyset clear singleunit
	}
end

/*
	returns in r():
	Scalars:
		r(stages)	number of sampling stages	or missing
	Macros:
		r(wtype)	"pweight" or "iweight"		or ""
		r(wexp)		"= <varname>"			or ""
		r(vce)		vcetype				or ""
		r(mse)		"mse"				or ""
		r(brrweight)	varlist				or ""
		r(fay)		#				or ""
		r(jkrweight)	varlist				or ""
		r(poststrata)	poststrata varname		or ""
		r(postweight)	postweight varname		or ""
		r(su#)		stage # ssu varname		or ""
		r(strata#)	stage # strata varname		or ""
		r(fpc#)		stage # fpc varname		or ""
		r(settings)	svyset args to reproduce current settings
*/
program Get, rclass
	syntax [, eclass]
	if "`eclass'" != "" {
		if "`e(prefix)'" != "svy" {
			di as err "svy estimation results not found"
			exit 301
		}
		GetEclass
		return add
		exit
	}
	local version : char _dta[_svy_version]
	capture confirm integer number `version'
	if c(rc) {
		GetOld
		return add
		exit
	}
	local is_set 0
	// get the global options
	_svyset get wvar
	if `"`r(wvar)'"' != "" {
		local wt `"[`r(wtype)'`r(wexp)']"'
		return add
		local is_set 1
	}
	_svyset get brr
	if "`r(brrweight)'" != "" {
		local gsets `"`gsets' brrweight(`r(brrweight)')"'
		return add
		local is_set 1
	}
	_svyset get fay
	if !inlist("`r(fay)'", "", "0") {
		local gsets `"`gsets' fay(`r(fay)')"'
		return add
		local is_set 1
	}
	_svyset get jkr
	if "`r(jkrweight)'" != "" {
		local gsets `"`gsets' jkrweight(`r(jkrweight)')"'
		return add
		local is_set 1
	}
	_svyset get posts
	if `"`r(poststrata)'`r(postweight)'"' != "" {
		local gsets `"`gsets' poststrata(`r(poststrata)')"'
		local gsets `"`gsets' postweight(`r(postweight)')"'
		return add
		local is_set 1
	}
	_svyset get singleunit
	if `"`r(singleunit)'"' != "" {
		local gsets `"`gsets' singleunit(`r(singleunit)')"'
		return add
		local is_set 1
	}
	else	local gsets `"`gsets' singleunit(missing)"'
	_svyset get vce
	if `"`r(vce)'"' != "" {
		local gsets `"`gsets' vce(`r(vce)')"'
		if "`r(mse)'" != "" {
			local gsets `"`gsets' mse"'
		}
		return add
		local is_set 1
	}
	else {
		local gsets `"`gsets' vce(linearized)"'
		return local vce linearized
	}

	// get the stage specific settings
	local stages : char _dta[_svy_stages]
	capture confirm integer number `stages'
	if c(rc) {
		local stages 0
	}
	else	local is_set 1
	local comma ","
	forval i = 1/`stages' {
		local comma ","
		_svyset get su `i'
		local su `r(su`i')'
		if `"`su'"' != "" {
			local sets "`sets'`oror'`su'"
			if `"`su'"' == "_n" {
				if `i' != `stages' {
					di as err "invalid use of _n; " ///
"observations can only be sampled in the final stage"
					exit 198
				}
				local su
			}
			else	return add
		}
		else {
			di as err ///
"invalid survey characteristics;" _n ///
"sampling unit variable is not set for stage `i' of `stages' stages"
			exit 459
		}
		if "`wt'" != "" {
			local sets "`sets' `wt'"
			local wt
		}
		_svyset get strata `i'
		local strata `r(strata`i')'
		if `"`strata'"' != "" {
			return add
			local sets `"`sets'`comma' strata(`strata')"'
			local comma
		}
		_svyset get fpc `i'
		local fpc `r(fpc`i')'
		if `"`fpc'"' != "" {
			return add
			local sets `"`sets'`comma' fpc(`fpc')"'
			local comma
		}
		else if `i' != `stages' {
			di as err ///
"invalid survey characteristics;" _n ///
"FPC variable is not set for stage `i' of `stages' stages"
			exit 459
		}
		local oror " || "
	}

	if `is_set' == 0 {
		return clear
		local sets ", clear"
	}
	else {
		if `"`gsets'"' != "" {
			local sets `"`sets'`comma' `:list retok gsets'"'
		}
		if `"`return(singleunit)'"' == "" {
			return local singleunit missing
		}
	}
	return local settings `"`sets'"'
	if `stages' {
		return scalar stages = `stages'
	}
end

program GetEclass, rclass
	// get the global options
	if `"`e(wexp)'"' != "" {
		local wt `"[`e(wtype)'`e(wexp)']"'
		return local wtype `"`e(wtype)'"'
		return local wexp  `"`e(wexp)'"'
		local wexp `"`e(wexp)'"'
		gettoken equal wvar : wexp
		return local wvar  `"`:list retok wvar'"'
	}
	if "`e(brrweight)'" != "" {
		local gsets `"`gsets' brrweight(`e(brrweight)')"'
		return local brrweight `e(brrweight)'
	}
	if !inlist("`e(fay)'", "", "0") {
		local gsets `"`gsets' fay(`e(fay)')"'
		return scalar fay = e(fay)
	}
	if "`e(jkrweight)'" != "" {
		local gsets `"`gsets' jkrweight(`e(jkrweight)')"'
		return local jkrweight `e(jkrweight)'
	}
	if `"`e(poststrata)'`e(postweight)'"' != "" {
		local gsets `"`gsets' poststrata(`e(poststrata)')"'
		local gsets `"`gsets' postweight(`e(postweight)')"'
		return local poststrata `e(poststrata)'
		return local postweight `e(postweight)'
	}
	if `"`e(singleunit)'"' != "" {
		local gsets `"`gsets' singleunit(`e(singleunit)')"'
		return local singleunit `e(singleunit)'
	}
	else {
		local gsets `"`gsets' singleunit(missing)"'
		return local singleunit missing
	}
	if `"`e(vce)'"' != "" {
		local gsets `"`gsets' vce(`e(vce)')"'
		if "`e(mse)'" != "" {
			local gsets `"`gsets' mse"'
			return local mse mse
		}
		return local vce `"`e(vce)'"'
	}

	// get the stage specific settings
	local stages = e(stages)
	capture confirm integer number `stages'
	if c(rc) {
		local stages 0
	}
	local comma ","
	forval i = 1/`stages' {
		local comma ","
		local su `e(su`i')'
		if `"`su'"' != "" {
			return local su`i' `su'
		}
		else	local su _n
		local sets "`sets'`oror'`su'"
		if "`wt'" != "" {
			local sets "`sets' `wt'"
			local wt
		}
		local strata `e(strata`i')'
		if `"`strata'"' != "" {
			return local strata`i' `strata'
			local sets `"`sets'`comma' strata(`strata')"'
			local comma
		}
		local fpc `e(fpc`i')'
		if `"`fpc'"' != "" {
			return local fpc`i' `fpc'
			local sets `"`sets'`comma' fpc(`fpc')"'
			local comma
		}
		local oror " || "
	}

	if `"`gsets'"' != "" {
		local sets `"`sets'`comma' `:list retok gsets'"'
	}
	return local settings `"`sets'"'
	if `stages' {
		return scalar stages = `stages'
	}
end

program GetOld, rclass
	quietly svyset_8
	if "`r(_svy)'" != "set" {
		local sets ", clear"
	}
	else {
		local comma ","
		if "`r(psu)'" != "" {
			return local su1 `"`r(psu)'"'
			local sets `"`r(psu)'"'
		}
		else {
			local sets _n
		}
		if "`r(wtype)'" != "" {
			return local wtype `"`r(wtype)'"'
			return local wexp  `"`r(wexp)'"'
			local wexp `"`r(wexp)'"'
			gettoken equal wvar : wexp
			return local wvar  `"`:list retok wvar'"'
			local sets `"`sets' [`r(wtype)'`r(wexp)']"'
		}
		if "`r(strata)'" != "" {
			return local strata1 `"`r(strata)'"'
			local sets ///
			`"`sets'`comma' strata(`r(strata)')"'
			local comma
		}
		if "`r(fpc)'" != "" {
			return local fpc1 `"`r(fpc)'"'
			local sets ///
			`"`sets'`comma' fpc(`r(fpc)')"'
		}
		return local singleunit missing
		return scalar stages = 1
		return local vce "linearized"
		local vce "`comma' vce(linearized)"
	}
	return local settings `"`:list retok sets'`vce'"'
end

program Display
	syntax [, eclass ]
	Get, `eclass'
	if `"`r(settings)'"' == ", clear" {
		di as txt "no survey characteristics are set"
		exit
	}
	di
	// display global settings
	local c2 16
	local sfmt %13s
	if `"`r(wtype)'"' != "" {
		local wvar `"`r(wvar)'"'
		di as txt `sfmt' "`r(wtype)'" ":" _col(`c2') as res "`wvar'"
	}
	else {
		di as txt `sfmt' "pweight" ":" _col(`c2') "<none>"
	}
	if `"`r(vce)'"' != "" {
		di as txt `sfmt' "VCE" ":" _col(`c2') as res "`r(vce)'"
		if !inlist("`r(vce)'", "", "linearized") {
			if "`r(mse)'" == "" {
				local mseas as txt
				local onoff off
			}
			else {
				local mseas as res
				local onoff on
			}
			di as txt `sfmt' "MSE" ":" _col(`c2') `mseas' "`onoff'"
		}
	}
	else {
		di as txt `sfmt' "VCE" ":" _col(`c2') "linearized"
	}
	local colsets 1 `c2' `c2' 2
	if `"`r(brrweight)'"' != "" {
		di "{p2colset `colsets'}{...}"
		di as txt "{p2col:{space 4}brrweight:}" ///
		   as res "`r(brrweight)'{p_end}"
		di "{p2colreset}{...}"
	}
	if `"`r(fay)'"' != "" {
		di "{p2colset `colsets'}{...}"
		di as txt "{p2col:{space 10}fay:}" ///
		   as res `r(fay)' "{p_end}"
		di "{p2colreset}{...}"
	}
	if `"`r(jkrweight)'"' != "" {
		di "{p2colset `colsets'}{...}"
		di as txt "{p2col:{space 4}jkrweight:}" ///
		   as res "`r(jkrweight)'{p_end}"
		di "{p2colreset}{...}"
	}
	if `"`r(poststrata)'"' != "" {
		di as txt `sfmt' "Poststrata" ":" ///
			_col(`c2') as res "`r(poststrata)'"
	}
	if `"`r(postweight)'"' != "" {
		di as txt `sfmt' "Postweight" ":" ///
			_col(`c2') as res "`r(postweight)'"
	}
	if `"`r(singleunit)'"' != "" {
		di as txt `sfmt' "Single unit" ":" ///
			_col(`c2') as res "`r(singleunit)'"
	}
	if !missing(r(stages)) {
		local stages = r(stages)
	}
	else {
		local stages = 0
	}
	forval i = 1/`stages' {
		if `"`r(strata`i')'"' != "" {
			di as txt `sfmt' "Strata `i'" ":" ///
				_col(`c2') as res "`r(strata`i')'"
		}
		else { // if `i' == 1 {
			di as txt `sfmt' "Strata `i'" ":" ///
				_col(`c2') "<one>"
		}
		if !inlist(`"`r(su`i')'"', "", "_n") {
			di as txt `sfmt' "SU `i'" ":" ///
				_col(`c2') as res "`r(su`i')'"
		}
		else {
			di as txt `sfmt' "SU `i'" ":" ///
				_col(`c2') "<observations>"
		}
		if `"`r(fpc`i')'"' != "" {
			di as txt `sfmt' "FPC `i'" ":" ///
				_col(`c2') as res "`r(fpc`i')'"
		}
		else	di as txt `sfmt' "FPC `i'" ":" ///
				_col(`c2') as txt "<zero>"
	}
	di as txt "{p2colreset}{...}"
end

program ParseOld
	quietly svyset_8 `0'
	if "`r(psu)'" == "" | "`r(srs)'" != "" {
		local uni1 _n
	}
	else	local uni1 `r(psu)'
	if "`r(wtype)'" != "" {
		local wt [`r(wtype)'`r(wexp)']
	}
	if "`r(strata)'" != "" {
		local strata strata(`strata')
	}
	if "`r(fpc)'" != "" {
		local fpc fpc(`fpc')
	}
	c_local 0 `"`psu' `wt', `strata' `fpc'"'
end

program JKnifeVChars, sclass
	capture noisily				///
	syntax [varlist(default=none)] [,	///
		STRatum(numlist integer)	///
		FPC(numlist >=0)		///
		MULTiplier(numlist >0)		///
		reset				///
	]
	local nvar  : word count `varlist'
	local nstr  : word count `stratum'
	local nfpc  : word count `fpc'
	local nmult : word count `multiplier'
	local invalid `c(rc)'
	if `nvar' == 0 & ( `nstr'`nfpc'`nmult' | "`reset'" != "") {
		jkrInvalid
	}
	if `nstr' > 1 & `nstr' != `nvar' {
		di as err "invalid number of stratum identifiers"
		if `nstr' < `nvar' {
			jkrInvalid 103
		}
		else	jkrInvalid 102
	}
	if `nfpc' > 1 & `nfpc' != `nvar' {
		di as err "invalid number of FPC values"
		if `nfpc' < `nvar' {
			jkrInvalid 103
		}
		else	jkrInvalid 102
	}
	if `nmult' > 1 & `nmult' != `nvar' {
		di as err "invalid number of jackknife multiplier values"
		if `nmult' < `nvar' {
			jkrInvalid 103
		}
		else	jkrInvalid 102
	}

	local noreset = "`reset'" == ""
	if `nstr'`nfpc'`nmult' | `"`reset'"' != "" {
		local str `"`stratum'"'
		if `nstr' > 1 {
			local strlist `"`stratum'"'
			local stratum
		}
		if `nfpc' > 1 {
			local fpclist `"`fpc'"'
		}
		local mult  `"`multiplier'"'
		if `nmult' > 1 {
			local multlist `"`multiplier'"'
			local multiplier
		}
		forval i = 1/`nvar' {
			local var : word `i' of `varlist'
			if `nstr' > 1 {
				local str : word `i' of `strlist'
			}
			if `nfpc' > 1 {
				local fpc : word `i' of `fpclist'
			}
			if `nmult' > 1 {
				local mult : word `i' of `multlist'
			}
			local char : char `var'[jk_stratum]
			if `noreset' ///
			 & !inlist(`"`char'"',"","`str'") {
				di as err "{p 0 0 2}" ///
"characteristic `var'[jk_stratum] already exists " ///
"and is different from the value supplied to stratum(){p_end}"
				exit 459
			}
			else	char `var'[jk_stratum] `str'
			local char : char `var'[jk_fpc]
			if `noreset' ///
			 & !inlist(`"`char'"',"","`fpc'") {
				di as err "{p 0 0 2}" ///
"characteristic `var'[jk_fpc] already exists " ///
"and is different from the value supplied to fpc(){p_end}"
				exit 459
			}
			else	char `var'[jk_fpc] `fpc'
			local char : char `var'[jk_multiplier]
			if `noreset' ///
			 & !inlist(`"`char'"',"","`mult'") {
				di as err "{p 0 0 2}" ///
"characteristic `var'[jk_multiplier] already exists " ///
"and is different from the value supplied to multiplier(){p_end}"
				exit 459
			}
			else	char `var'[jk_multiplier] `mult'
		}
	}
	sreturn local varlist `"`varlist'"'
end

program jkrInvalid
	args rc
	di as err "option jkrweight() incorrectly specified"
	if "`rc'" == "" {
		exit 198
	}
	else	exit `rc'
end

exit
